'''
 Licensed to the Apache Software Foundation (ASF) under one
 or more contributor license agreements.  See the NOTICE file
 distributed with this work for additional information
 regarding copyright ownership.  The ASF licenses this file
 to you under the Apache License, Version 2.0 (the
 "License"); you may not use this file except in compliance
 with the License.  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing,
 software distributed under the License is distributed on an
 "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
'''

import aliyunCliParser
import aliyunOpenApiData
import aliyunExtensionCliHandler
import response
import aliyunCliHelp
import aliyunCliUpgrade
import aliyunCompleter
import sys


class AliyunCommandLine:
    def __init__(self):
        self.parser = aliyunCliParser.aliyunCliParser()
        self.handler = aliyunOpenApiData.aliyunOpenApiDataHandler()
        self.extensionHandler = aliyunExtensionCliHandler.aliyunExtensionCliHandler()
        self.helper = aliyunCliHelp.aliyunCliHelper()
        self.args = sys.argv[1:]
        self.completer = aliyunCompleter.Completer()

    def main(self):

        # fetch command
        cmd = self.parser.getCliCmd()
        extensionCmdList = self.extensionHandler.getAllExtensionCommands()

        if cmd in extensionCmdList:
            self.handlerExtensionCmd(cmd)
            return

        # fetch operation
        operation = self.parser.getCliOperation()

        # fetch paramlist
        keyValues = self.parser._getKeyValues()
        
        outPutFormat = self.parser.getOutPutFormat(keyValues)
        if outPutFormat is None or len(outPutFormat) == 0:
            outPutFormat = self.extensionHandler.getUserFormat()
            if outPutFormat is None or outPutFormat == "":
                outPutFormat = 'json'
        else:
            outPutFormat = outPutFormat[0]

        secureRequest = self.parser.getSecureChoice(keyValues)

        if self.handler.isEndPointOperation(operation):
            keyValues = self.parser.getOpenApiKeyValues(keyValues)
            self.handler.handleEndPointOperation(cmd,operation,keyValues)
            return

        if self.handler.isAvailableCmd(cmd):
            # fetch getversion
            if self.handler.isNonStandardSdkCmd(cmd):
                self.handler.nonStandardSdkCmdHandle(cmd)
                return
            version=self.handler.getSdkVersion(cmd,keyValues)
            if version is None:
                return

            # here handler the openapi cmd
            if self.handler.isAvailableOperation(cmd, operation,version): # cmd and operation both are right
                instanceAndClassName=self.handler.getInstanceByCmdOperation(cmd, operation,version)
                if instanceAndClassName is not None and len(instanceAndClassName)==2:
                    cmdInstance = instanceAndClassName[0]
                    className = instanceAndClassName[1]
                    if cmdInstance is not None and className is not None:
                        if self.showInstanceAttribute(cmd, operation, className):
                            return
                        # here should handle the keyValues first
                        keyValues = self.parser.getOpenApiKeyValues(keyValues)
                        if self.handler.needSetDefaultRegion(cmdInstance, keyValues):
                            keyValues["RegionId"] = [self.extensionHandler.getUserRegion()]
                        #check necessaryArgs as:accesskeyid accesskeysecret regionId
                        if not self.handler.hasNecessaryArgs(keyValues):
                            print 'accesskeyid/accesskeysecret/regionId is absence'
                            return
                        result = self.handler.getResponse(cmd,operation,className,cmdInstance,keyValues,secureRequest)
                        if result is None:
                            return
                        self.handler.responseOptimize(result,cmd,operation)
                        if("Code" in result):
                            response.display_response("error", result, "json")
                        else:
                            response.display_response(operation, result, outPutFormat,keyValues)
                    else:
                        print 'aliyuncli internal error, please contact: haowei.yao@alibaba-inc.com'
            elif self.handler.isAvailableExtensionOperation(cmd, operation):
                if self.args.__len__() >= 3 and self.args[2] == 'help':
                    import commandConfigure
                    configure = commandConfigure.commandConfigure()
                    configure.showExtensionOperationHelp(cmd, operation)
                else:
                    self.extensionHandler.handlerExtensionOperation(cmd,operation,version,secureRequest)
                # self.extensionHandler.handlerExtensionOperation(cmd,operation,version)
            else:
                # cmd is right but operation is not right
                self.helper.showOperationError(cmd, operation)
        else:
            self.helper.showCmdError(cmd)



    def handlerExtensionCmd(self, cmd):
        self.extensionHandler.handlerExtensionCmd(cmd)

    def showInstanceAttribute(self, cmd, operation, classname):
        if self.args.__len__() >= 3 and self.args[2] == "help":
            self.helper.showParameterError(cmd, operation, self.completer._help_to_show_instance_attribute(classname))
            #print self.completer._help_to_show_instance_attribute(cmdInstance)
            return True
        return False











'''
 Licensed to the Apache Software Foundation (ASF) under one
 or more contributor license agreements.  See the NOTICE file
 distributed with this work for additional information
 regarding copyright ownership.  The ASF licenses this file
 to you under the Apache License, Version 2.0 (the
 "License"); you may not use this file except in compliance
 with the License.  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing,
 software distributed under the License is distributed on an
 "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
'''
__author__ = 'xixi.xxx'

import aliyunCliHelp
import aliyunOpenApiData

class commandConfigure:
    def __init__(self):
        self.ecs = ecs()
        self.rds = rds()
        self.main_options = ['output', 'AccessKeyId', 'AccessKeySecret', 'Endpoint']
        self.helper = aliyunCliHelp.aliyunCliHelper()
        self.open_api_headler = aliyunOpenApiData.aliyunOpenApiDataHandler()

    def getExtensionOptions(self, cmd, operation):
        if cmd is None or operation is None:
            return None
        if operation in aliyunOpenApiData.version_cmds:
            return None
        if cmd.lower() == 'rds':
            _rds = rds()
            return _rds.extensionOptions[operation]
        if cmd.lower() == 'ecs':
            _ecs = ecs()
            return _ecs.extensionOptions[operation]
        return None

    def getExtensionOperations(self, cmd):
        return self.open_api_headler.getExtensionOperationsFromCmd(cmd)
    #
    # this api will return the options for each extension operations, such as "ecs importInstance" "rds ExportDBInstance"
    #
    def showExtensionOperationHelp(self, cmd, operation):
        parameterList = list()
        if cmd.lower() == "rds":
            self.appendList(parameterList, self.main_options)
            if operation.lower() == self.rds.exportDBInstance.lower():
                # _mlist = self.rds.extensionOptions[self.rds.exportDBInstance]
                self.appendList(parameterList, self.rds.extensionOptions[self.rds.exportDBInstance])
            if operation.lower() == self.rds.importDBInstance.lower():
                # print "haha", (self.rds.extensionOptions[self.rds.importDBInstance])
                # parameterList.append(self.rds.extensionOptions[self.rds.importDBInstance])
                self.appendList(parameterList, self.rds.extensionOptions[self.rds.importDBInstance])

        if cmd.lower() == "ecs":
            self.appendList(parameterList, self.main_options)
            if operation.lower() == self.ecs.exportInstance.lower():
                self.appendList(parameterList, self.ecs.extensionOptions[self.ecs.exportInstance])
            if operation.lower() == self.ecs.importInstance.lower():
                self.appendList(parameterList, self.ecs.extensionOptions[self.ecs.importInstance])

        self.helper.showParameterError(cmd, operation, parameterList)

    def appendList(self, parameterList, optionList):
        for item in optionList:
            parameterList.append(item)

class rds:
    cmdName = 'Rds'
    exportDBInstance = 'ExportDBInstance'
    importDBInstance = 'ImportDBInstance'
    extensionOperations = [exportDBInstance, importDBInstance]
    extensionOptions = {exportDBInstance:['DBInstanceId','OwnerAccount','OwnerId','ResourceOwnerAccount','filename'],
                        importDBInstance:['count','filename']}

class ecs:
    cmdName = 'Ecs'
    exportInstance = 'ExportInstance'
    importInstance = 'ImportInstance'
    extensionOperations = [exportInstance, importInstance]
    extensionOptions = {exportInstance:['InstanceId','OwnerAccount','OwnerId','ResourceOwnerAccount','filename'],
                        importInstance:['count','filename']}

if __name__ == '__main__':
    # print type(rds.extensionOperations)
    # print type(rds.extensionOptions)
    # print rds.extensionOptions['ll']
    configure = commandConfigure()
    print configure.showExtensionOperationHelp("ecs", "ExportInstance")



'''
 Licensed to the Apache Software Foundation (ASF) under one
 or more contributor license agreements.  See the NOTICE file
 distributed with this work for additional information
 regarding copyright ownership.  The ASF licenses this file
 to you under the Apache License, Version 2.0 (the
 "License"); you may not use this file except in compliance
 with the License.  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing,
 software distributed under the License is distributed on an
 "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
'''
# -*- coding:utf-8 -*-

import sys
import json
import text
from table import MultiTable
import jmespath


class Response(object):

    def __init__(self,args):
        self.args = args
    def __call__(self, command,response, stream=None):
        if stream is None:
            stream = sys.stdout
        if _has_filter_param(self.args)[0]:
            filter_value =_has_filter_param(self.args)[1]
            expression = jmespath.compile(filter_value)
            response = expression.search(response)
        try:
            self._format_response(command, response, stream)
        except IOError as e:
            pass
        finally:
            self._flush_stream(stream)
    def _flush_stream(self, stream):
        try:
            stream.flush()
        except IOError:
            pass


class JSONResponse(Response):
    def _format_response (self, command,response,stream=None):
        if stream is None :
            stream = sys.stdout
        if response:
            json.dump(response,stream, indent=4)
            stream.write('\n')


class TextResponse (Response):
    def __call__(self, command, response, stream=None):
        if stream is None:
            stream = sys.stdout
        try:
            self._format_response(response, stream)
        finally:
            # flush is needed to avoid the "close failed in file object
            # destructor" in python2.x (see http://bugs.python.org/issue11380).
            self._flush_stream(stream)

    def _format_response(self, response, stream):
        if _has_filter_param(self.args)[0]:
            filter_value =_has_filter_param(self.args)[1]
            expression = jmespath.compile(filter_value)
            response = expression.search(response)
        text.format_text(response, stream)

class TableResponse (Response):
    def __init__(self, args, table=None):
        super(TableResponse, self).__init__(args)
        self.table = MultiTable(initial_section=False,
                                    column_separator='|')

    def _format_response(self, command, response, stream):
        if self._build_table(command, response):
            try:
                self.table.render(stream)
            except IOError:
                # If they're piping stdout to another process which exits before
                # we're done writing all of our output, we'll get an error about a
                # closed pipe which we can safely ignore.
                pass

    def _build_table(self, title, current, indent_level=0):
        if not current:
            return False
        if title is not None:
            self.table.new_section(title, indent_level=indent_level)
        if isinstance(current, list):
            if isinstance(current[0], dict):
                self._build_sub_table_from_list(current, indent_level, title)
            else:
                for item in current:
                    if self._scalar_type(item):
                        self.table.add_row([item])
                    elif all(self._scalar_type(el) for el in item):
                        self.table.add_row(item)
                    else:
                        self._build_table(title=None, current=item)
        if isinstance(current, dict):
            # Render a single row section with keys as header
            # and the row as the values, unless the value
            # is a list.
            self._build_sub_table_from_dict(current, indent_level)
        return True

    def _build_sub_table_from_dict(self, current, indent_level):
        # Render a single row section with keys as header
        # and the row as the values, unless the value
        # is a list.
        headers, more = self._group_scalar_keys(current)
        if len(headers) == 1:
            # Special casing if a dict has a single scalar key/value pair.
            self.table.add_row([headers[0], current[headers[0]]])
        elif headers:
            self.table.add_row_header(headers)
            self.table.add_row([current[k] for k in headers])
        for remaining in more:
            self._build_table(remaining, current[remaining],indent_level=indent_level + 1)

    def _build_sub_table_from_list(self, current, indent_level, title):
        headers, more = self._group_scalar_keys_from_list(current)
        self.table.add_row_header(headers)
        first = True
        for element in current:
            if not first and more:
                self.table.new_section(title,
                                       indent_level=indent_level)
                self.table.add_row_header(headers)
            first = False
            # Use .get() to account for the fact that sometimes an element
            # may not have all the keys from the header.
            self.table.add_row([element.get(header, '') for header in headers])
            for remaining in more:
                # Some of the non scalar attributes may not necessarily
                # be in every single element of the list, so we need to
                # check this condition before recursing.
                if remaining in element:
                    self._build_table(remaining, element[remaining],
                                    indent_level=indent_level + 1)

    def _scalar_type(self, element):
        return not isinstance(element, (list, dict))

    def _group_scalar_keys_from_list(self, list_of_dicts):
        # We want to make sure we catch all the keys in the list of dicts.
        # Most of the time each list element has the same keys, but sometimes
        # a list element will have keys not defined in other elements.
        headers = set()
        more = set()
        for item in list_of_dicts:
            current_headers, current_more = self._group_scalar_keys(item)
            headers.update(current_headers)
            more.update(current_more)
        headers = list(sorted(headers))
        more = list(sorted(more))
        return headers, more

    def _group_scalar_keys(self, current):
        # Given a dict, separate the keys into those whose values are
        # scalar, and those whose values aren't.  Return two lists,
        # one is the scalar value keys, the second is the remaining keys.
        more = []
        headers = []
        for element in current:
            if self._scalar_type(current[element]):
                headers.append(element)
            else:
                more.append(element)
        headers.sort()
        more.sort()
        return headers, more


def _has_filter_param(args):
    has = False
    param =None
    if isinstance(args,dict):
        value = args.get('filter')
        if isinstance(value,list) and len(value)>0:
            param=value[0]
            param = param.strip()
            if len(param) >0:
                has=True
    return [has,param]



def get_response (output_type,parsed_args):
    if output_type == None :
        output_type = 'JSON'
    output_type = output_type.lower()
    if output_type == 'json':
            return JSONResponse(parsed_args)
    elif output_type  == 'text':
        return TextResponse(parsed_args)
    elif output_type == 'table':
        return TableResponse(parsed_args)
    raise ValueError("Unknown output type: %s" % output_type)



def display_response(command, response,output,parsed_globals=None):
    if output is None:
        output = 'JSON'
    formatter = get_response(output, parsed_globals)
    formatter(command, response)





__author__ = 'xixi.xxx'
import os,sys
parentdir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0,parentdir)
import aliyunExtensionCliHandler
class ProfileCmd:
    useProfile = 'useProfile'
    addProfile = 'addProfile'
    name = '--name'
class ProfileHandler:
    def __init__(self):
        self.extensionCliHandler = aliyunExtensionCliHandler.aliyunExtensionCliHandler()

    def getProfileHandlerCmd(self):
        return [ProfileCmd.useProfile, ProfileCmd.addProfile]

    def getProfileHandlerOptions(self):
        return ['--name']

    def handleProfileCmd(self, cmd, keyValues):
        if cmd.lower() == ProfileCmd.useProfile.lower(): # confirm command is right
            #check --name is valid
            if keyValues.has_key(ProfileCmd.name) and len(keyValues[ProfileCmd.name]) > 0:
                _value = keyValues[ProfileCmd.name][0] # use the first value
                self.extensionCliHandler.setUserProfile(_value)
            else:
                print "Do your forget profile name? please use \'--name\' and add the profile name."
        else:
            print "[", cmd, "] is not right, do you mean "+ProfileCmd.useProfile+" ?"

    def addProfileCmd(self, cmd, keyValues):
        userKey = ''
        userSecret = ''
        newProfileName = ''
        if cmd.lower() == ProfileCmd.addProfile.lower(): # confirm command is right
            #check --name is valid
            if keyValues.has_key(ProfileCmd.name) and len(keyValues[ProfileCmd.name]) > 0:
                _value = keyValues[ProfileCmd.name][0] # check the first value
                # only input key and secret
                newProfileName = _value
            else:
                # need input profilename key and value
                newProfileName = raw_input("profile name [None]: ")
            userKey = raw_input("Aliyun Access Key ID [None]: ")
            userSecret = raw_input("Aliyun Secret Access Key [None]: ")
            _credentialsPath = os.path.join(self.extensionCliHandler.aliyunConfigurePath,self.extensionCliHandler.credentials)
            if os.path.exists(_credentialsPath):
                f = open(_credentialsPath, 'a')
                try:
                    content = "\n["+newProfileName+"]\naliyun_access_key_id="+userKey+"\naliyun_secret_access_key="+userSecret+"\n"
                    f.write(content)
                finally:
                    f.close()
        else:
            print "[", cmd, "] is not right, do you mean "+ProfileCmd.addProfile+" ?"


if __name__ == "__main__":
    handler = ProfileHandler()
    handler.handleProfileCmd("useprofile", {'--name':["profile444"]})
    print handler.extensionCliHandler.getUserProfile()
    handler.addProfileCmd("addProfile", {})
    handler.addProfileCmd("addProfile", {'--name':["profile2222"]})


'''
 Licensed to the Apache Software Foundation (ASF) under one
 or more contributor license agreements.  See the NOTICE file
 distributed with this work for additional information
 regarding copyright ownership.  The ASF licenses this file
 to you under the Apache License, Version 2.0 (the
 "License"); you may not use this file except in compliance
 with the License.  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing,
 software distributed under the License is distributed on an
 "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
'''

import os
import platform
import aliyunCliHelp
import aliyunCliConfiugre
import advance.userProfileHandler
import advance.userConfigHandler
import configure
import advance.ecsExportHandler
import advance.ecsImportHandler
import aliyunCliParser
from __init__ import __version__ as version
class color:
    if platform.system() == "Windows":
        purple = ''
        cyan = ''
        darkcyan = ''
        blue = ''
        green = ''
        yellow = ''
        red  = ''
        bold = ''
        underline = ''
        end = ''
    else:
        purple = '\033[95m'
        cyan = '\033[96m'
        darkcyan = '\033[36m'
        blue = '\033[94m'
        green = '\033[92m'
        yellow = '\033[93m'
        red  = '\033[91m'
        bold = '\033[1m'
        underline = '\033[4m'
        end = '\033[0m'

class AliyunCredentials:
    aliyun_access_key_id = "aliyun_access_key_id"
    aliyun_access_key_secret = "aliyun_access_key_secret"

class aliyunExtensionCliHandler:
    def __init__(self):
        self._version = version
        self.home = ".aliyuncli"
        self.configure = "configure"
        self.credentials = "credentials"
        self.aliyunConfigurePath = os.path.join(self.findConfigureFilePath(),self.home)
        self.globalConfigure = aliyunCliConfiugre.configure()
        self.parser = aliyunCliParser.aliyunCliParser()
    def getExtensionCmd(self):
        return None

# this function will give all extension command defined by us
    def getAllExtensionCommands(self):
        cmds = list()
        cmds = ['-h', '--help', 'help', 'configure', '--version', 'version', 'useprofile', 'addprofile','showconfig']
        return cmds
# this function will handler extension operation
    def handlerExtensionOperation(self,cmd,operation,version,secureRequest = False):
        defaultOperations=['configversion','showversions']
        if operation.lower() in defaultOperations:
            import aliyunSdkConfigure
            aliyunSdkConfigure.handleSdkVersion(cmd,operation,version)
        if cmd.lower() == 'ecs':
            import commandConfigure
            import advance.ecsExportHandler
            import advance.ecsImportHandler
            ecsConfigure = commandConfigure.ecs()
            _ecsImportHandler = advance.ecsImportHandler.EcsImportHandler()
            _ecsExportHandler = advance.ecsExportHandler.EcsExportHandler()
            if  operation.lower() == ecsConfigure.importInstance.lower():
                _ecsImportHandler.ImportInstance(cmd,operation,version,secureRequest)
            elif operation.lower() == ecsConfigure.exportInstance.lower():
                _ecsExportHandler.exportInstance(cmd,operation,version,secureRequest)

        if cmd.lower() == 'rds':
            import commandConfigure
            import advance.rdsExportHandler
            import advance.rdsImportHandler
            rdsConfigure = commandConfigure.rds()
            rdsExportDBInstanceHandler = advance.rdsExportHandler.RdsExportDBInstanceHanlder()
            rdsImportDBInstanceHanlder = advance.rdsImportHandler.RdsImportDBInstanceHandler()
            if operation.lower() == rdsConfigure.exportDBInstance.lower():
                rdsExportDBInstanceHandler.exportDBInstance(cmd,operation,version,secureRequest)
            elif operation.lower() == rdsConfigure.importDBInstance.lower():
                rdsImportDBInstanceHanlder.importInstance(cmd, operation, version,secureRequest)
		
# this function will handler extension command
    def handlerExtensionCmd(self, cmd):
        _cmd = self.parser._getCommand()
        _keyvalues = self.parser._getKeyValues()
        operation = self.parser._getOperations()
        if cmd.lower() in ['help', '-h', '--help']:
            self.showAliyunCliHelp()
        if cmd.lower() == 'configure':
            configure.handleConfigure(cmd,operation)
        if cmd.lower() == 'useprofile':
            _profileHandler = advance.userProfileHandler.ProfileHandler()
            _profileHandler.handleProfileCmd(_cmd, _keyvalues)
        if cmd.lower() == 'addprofile':
            _profileHandler = advance.userProfileHandler.ProfileHandler()
            _profileHandler.addProfileCmd(_cmd, _keyvalues)
        if cmd.lower() in ['--version', 'version']:
            self.showCurrentVersion()
        if cmd.lower() == 'showconfig':
            _configHandler = advance.userConfigHandler.ConfigHandler()
            _configHandler.showConfig()
		 

# this api will show help page when user input aliyuncli help(-h or --help)
    def showAliyunCliHelp(self):
        print color.bold+"ALIYUNCLI()"+color.end
        print color.bold+"\nNAME"+color.end
        print "\taliyuncli -"
        print color.bold+"\nDESCRIPTION"+color.end
        print "\tThe Aliyun Command Line Interface is a unified tool to manage your aliyun services. "
        print color.bold+"\nSYNOPSIS"+color.end
        print "\taliyuncli <command> <operation> [options and parameters]"
        print "\n\taliyuncli has supported command completion now. The detail you can check our site."
        print color.bold+"OPTIONS"+color.end
        print color.bold+"\tconfigure"+color.end
        print "\n\tThis option will help you save the key and secret and your favorite output format (text, json or table)"
        print color.bold+"\n\t--output"+color.end+" (string)"
        print "\n\tThe formatting style for command output."
        print "\n\to json"
        print "\n\to text"
        print "\n\to table"
        
        print color.bold+"\n\t--secure"+color.end
        print "\n\tMaking secure requests(HTTPS) to service"
        
        print color.bold+"\nAVAILABLE SERVICES"+color.end
        print "\n\to ecs"
        print "\n\to ess"
        print "\n\to mts"
        print "\n\to rds"
        print "\n\to slb"

    def showCurrentVersion(self):
        print self._version

    def findConfigureFilePath(self):
        homePath = ""
        if platform.system() == "Windows":
            homePath = os.environ['HOMEPATH']
            pass
        else:
            homePath = os.environ['HOME']
            pass
        return homePath
    def getUserRegion(self):
        handler = configure.AliyunConfig()
        return handler._getUserRegion()

    def getUserFormat(self):
        handler = configure.AliyunConfig()
        return handler._getUserFormat()
    def getUserKey(self):
        handler = configure.AliyunConfig()
        return handler._getUserKey()

    def getUserSecret(self):
        handler = configure.AliyunConfig()
        return  handler._getUserSecret()

    def getPort(self):
        handler = configure.AliyunConfig()
        return  handler._getPort()
if __name__ == "__main__":
    pass
# this function will show aliyun cli help



__author__ = 'zhaoyang.szy'
import os,sys
import response 
parentdir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0,parentdir)
import aliyunExtensionCliHandler
class ConfigCmd:
    showConfig = 'showConfig'
    importConfig = 'importConfig'
    exportConfig = 'exportConfig'
    name = '--filename'
class ConfigHandler:
    def __init__(self):
        self.extensionCliHandler = aliyunExtensionCliHandler.aliyunExtensionCliHandler()
    def getConfigHandlerCmd(self):
        return [ConfigCmd.showConfig,ConfigCmd.importConfig,ConfigCmd.exportConfig]

    def getConfigHandlerOptions(self):
        return [ConfigCmd.name]
				
    def showConfig(self):
        _credentialsPath = os.path.join(self.extensionCliHandler.aliyunConfigurePath,self.extensionCliHandler.credentials)
        _configurePath = os.path.join(self.extensionCliHandler.aliyunConfigurePath,self.extensionCliHandler.configure)
        config = dict()
        configContent = dict() 
	credentialsContent = dict ()
	if os.path.exists(_configurePath):
            for line in open(_configurePath):
                line = line.strip('\n')
                if line.find('=') > 0:
                    list = line.split("=",1)
		    configContent[list[0]] = list[1]
		else: 
		    pass
	config['configure'] = configContent
	if os.path.exists(_credentialsPath):
	    for line in open(_credentialsPath):
                line = line.strip('\n')
                if line.find('=') > 0:
                    list = line.split("=",1)
		    credentialsContent[list[0]] = list[1]
		else: 
		    pass 
	config ['credentials'] = credentialsContent
	response.display_response("showConfigure",config,'table')
    def importConfig():
        pass
    def exportConfig():
        pass
	


if __name__ == "__main__":
    handler = ConfigHandler()
    handler.showConfig()



'''
 Licensed to the Apache Software Foundation (ASF) under one
 or more contributor license agreements.  See the NOTICE file
 distributed with this work for additional information
 regarding copyright ownership.  The ASF licenses this file
 to you under the Apache License, Version 2.0 (the
 "License"); you may not use this file except in compliance
 with the License.  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing,
 software distributed under the License is distributed on an
 "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
'''

import six


def format_text(data, stream):
    _format_text(data, stream)


def _format_text(item, stream, identifier=None, scalar_keys=None):
    if isinstance(item, dict):
        _format_dict(scalar_keys, item, identifier, stream)
    elif isinstance(item, list):
        _format_list(item, identifier, stream)
    else:
        # If it's not a list or a dict, we just write the scalar
        # value out directly.
        stream.write(six.text_type(item))
        stream.write('\n')


def _format_list(item, identifier, stream):
    if not item:
        return
    if any(isinstance(el, dict) for el in item):
        all_keys = _all_scalar_keys(item)
        for element in item:
            _format_text(element, stream=stream, identifier=identifier,
                         scalar_keys=all_keys)
    elif any(isinstance(el, list) for el in item):
        scalar_elements, non_scalars = _partition_list(item)
        if scalar_elements:
            _format_scalar_list(scalar_elements, identifier, stream)
        for non_scalar in non_scalars:
            _format_text(non_scalar, stream=stream,
                         identifier=identifier)
    else:
        _format_scalar_list(item, identifier, stream)


def _partition_list(item):
    scalars = []
    non_scalars = []
    for element in item:
        if isinstance(element, (list, dict)):
            non_scalars.append(element)
        else:
            scalars.append(element)
    return scalars, non_scalars


def _format_scalar_list(elements, identifier, stream):
    if identifier is not None:
        for item in elements:
            stream.write('%s\t%s\n' % (identifier.upper(),
                                       item))
    else:
        # For a bare list, just print the contents.
        stream.write('\t'.join([six.text_type(item) for item in elements]))
        stream.write('\n')


def _format_dict(scalar_keys, item, identifier, stream):
    scalars, non_scalars = _partition_dict(item, scalar_keys=scalar_keys)
    if scalars:
        if identifier is not None:
            scalars.insert(0, identifier.upper())
        stream.write('\t'.join(scalars))
        stream.write('\n')
    for new_identifier, non_scalar in non_scalars:
        _format_text(item=non_scalar, stream=stream,
                     identifier=new_identifier)


def _all_scalar_keys(list_of_dicts):
    keys_seen = set()
    for item_dict in list_of_dicts:
        for key, value in item_dict.items():
            if not isinstance(value, (dict, list)):
                keys_seen.add(key)
    return list(sorted(keys_seen))


def _partition_dict(item_dict, scalar_keys):
    # Given a dictionary, partition it into two list based on the
    # values associated with the keys.
    # {'foo': 'scalar', 'bar': 'scalar', 'baz': ['not, 'scalar']}
    # scalar = [('foo', 'scalar'), ('bar', 'scalar')]
    # non_scalar = [('baz', ['not', 'scalar'])]
    scalar = []
    non_scalar = []
    if scalar_keys is None:
        # scalar_keys can have more than just the keys in the item_dict,
        # but if user does not provide scalar_keys, we'll grab the keys
        # from the current item_dict
        for key, value in sorted(item_dict.items()):
            if isinstance(value, (dict, list)):
                non_scalar.append((key, value))
            else:
                scalar.append(six.text_type(value))
    else:
        for key in scalar_keys:
            scalar.append(six.text_type(item_dict.get(key, '')))
        remaining_keys = sorted(set(item_dict.keys()) - set(scalar_keys))
        for remaining_key in remaining_keys:
            non_scalar.append((remaining_key, item_dict[remaining_key]))
    return scalar, non_scalar



'''
 Licensed to the Apache Software Foundation (ASF) under one
 or more contributor license agreements.  See the NOTICE file
 distributed with this work for additional information
 regarding copyright ownership.  The ASF licenses this file
 to you under the Apache License, Version 2.0 (the
 "License"); you may not use this file except in compliance
 with the License.  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing,
 software distributed under the License is distributed on an
 "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
'''

from __future__ import absolute_import

import functools
import operator
import sys
import types



# Useful for very coarse version differentiation.
PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3

if PY3:
    string_types = str,
    integer_types = int,
    class_types = type,
    text_type = str
    binary_type = bytes

    MAXSIZE = sys.maxsize
else:
    string_types = basestring,
    integer_types = (int, long)
    class_types = (type, types.ClassType)
    text_type = unicode
    binary_type = str

    if sys.platform.startswith("java"):
        # Jython always uses 32 bits.
        MAXSIZE = int((1 << 31) - 1)
    else:
        # It's possible to have sizeof(long) != sizeof(Py_ssize_t).
        class X(object):
            def __len__(self):
                return 1 << 31
        try:
            len(X())
        except OverflowError:
            # 32-bit
            MAXSIZE = int((1 << 31) - 1)
        else:
            # 64-bit
            MAXSIZE = int((1 << 63) - 1)
        del X


def _add_doc(func, doc):
    """Add documentation to a function."""
    func.__doc__ = doc


def _import_module(name):
    """Import module, returning the module after the last dot."""
    __import__(name)
    return sys.modules[name]


class _LazyDescr(object):

    def __init__(self, name):
        self.name = name

    def __get__(self, obj, tp):
        result = self._resolve()
        setattr(obj, self.name, result) # Invokes __set__.
        # This is a bit ugly, but it avoids running this again.
        delattr(obj.__class__, self.name)
        return result


class MovedModule(_LazyDescr):

    def __init__(self, name, old, new=None):
        super(MovedModule, self).__init__(name)
        if PY3:
            if new is None:
                new = name
            self.mod = new
        else:
            self.mod = old

    def _resolve(self):
        return _import_module(self.mod)

    def __getattr__(self, attr):
        _module = self._resolve()
        value = getattr(_module, attr)
        setattr(self, attr, value)
        return value


class _LazyModule(types.ModuleType):

    def __init__(self, name):
        super(_LazyModule, self).__init__(name)
        self.__doc__ = self.__class__.__doc__

    def __dir__(self):
        attrs = ["__doc__", "__name__"]
        attrs += [attr.name for attr in self._moved_attributes]
        return attrs

    # Subclasses should override this
    _moved_attributes = []


class MovedAttribute(_LazyDescr):

    def __init__(self, name, old_mod, new_mod, old_attr=None, new_attr=None):
        super(MovedAttribute, self).__init__(name)
        if PY3:
            if new_mod is None:
                new_mod = name
            self.mod = new_mod
            if new_attr is None:
                if old_attr is None:
                    new_attr = name
                else:
                    new_attr = old_attr
            self.attr = new_attr
        else:
            self.mod = old_mod
            if old_attr is None:
                old_attr = name
            self.attr = old_attr

    def _resolve(self):
        module = _import_module(self.mod)
        return getattr(module, self.attr)


class _SixMetaPathImporter(object):
    """
    A meta path importer to import six.moves and its submodules.

    This class implements a PEP302 finder and loader. It should be compatible
    with Python 2.5 and all existing versions of Python3
    """
    def __init__(self, six_module_name):
        self.name = six_module_name
        self.known_modules = {}

    def _add_module(self, mod, *fullnames):
        for fullname in fullnames:
            self.known_modules[self.name + "." + fullname] = mod

    def _get_module(self, fullname):
        return self.known_modules[self.name + "." + fullname]

    def find_module(self, fullname, path=None):
        if fullname in self.known_modules:
            return self
        return None

    def __get_module(self, fullname):
        try:
            return self.known_modules[fullname]
        except KeyError:
            raise ImportError("This loader does not know module " + fullname)

    def load_module(self, fullname):
        try:
            # in case of a reload
            return sys.modules[fullname]
        except KeyError:
            pass
        mod = self.__get_module(fullname)
        if isinstance(mod, MovedModule):
            mod = mod._resolve()
        else:
            mod.__loader__ = self
        sys.modules[fullname] = mod
        return mod

    def is_package(self, fullname):
        """
        Return true, if the named module is a package.

        We need this method to get correct spec objects with
        Python 3.4 (see PEP451)
        """
        return hasattr(self.__get_module(fullname), "__path__")

    def get_code(self, fullname):
        """Return None

        Required, if is_package is implemented"""
        self.__get_module(fullname)  # eventually raises ImportError
        return None
    get_source = get_code  # same as get_code

_importer = _SixMetaPathImporter(__name__)


class _MovedItems(_LazyModule):
    """Lazy loading of moved objects"""
    __path__ = []  # mark as package


_moved_attributes = [
    MovedAttribute("cStringIO", "cStringIO", "io", "StringIO"),
    MovedAttribute("filter", "itertools", "builtins", "ifilter", "filter"),
    MovedAttribute("filterfalse", "itertools", "itertools", "ifilterfalse", "filterfalse"),
    MovedAttribute("input", "__builtin__", "builtins", "raw_input", "input"),
    MovedAttribute("intern", "__builtin__", "sys"),
    MovedAttribute("map", "itertools", "builtins", "imap", "map"),
    MovedAttribute("range", "__builtin__", "builtins", "xrange", "range"),
    MovedAttribute("reload_module", "__builtin__", "imp", "reload"),
    MovedAttribute("reduce", "__builtin__", "functools"),
    MovedAttribute("shlex_quote", "pipes", "shlex", "quote"),
    MovedAttribute("StringIO", "StringIO", "io"),
    MovedAttribute("UserDict", "UserDict", "collections"),
    MovedAttribute("UserList", "UserList", "collections"),
    MovedAttribute("UserString", "UserString", "collections"),
    MovedAttribute("xrange", "__builtin__", "builtins", "xrange", "range"),
    MovedAttribute("zip", "itertools", "builtins", "izip", "zip"),
    MovedAttribute("zip_longest", "itertools", "itertools", "izip_longest", "zip_longest"),

    MovedModule("builtins", "__builtin__"),
    MovedModule("configparser", "ConfigParser"),
    MovedModule("copyreg", "copy_reg"),
    MovedModule("dbm_gnu", "gdbm", "dbm.gnu"),
    MovedModule("_dummy_thread", "dummy_thread", "_dummy_thread"),
    MovedModule("http_cookiejar", "cookielib", "http.cookiejar"),
    MovedModule("http_cookies", "Cookie", "http.cookies"),
    MovedModule("html_entities", "htmlentitydefs", "html.entities"),
    MovedModule("html_parser", "HTMLParser", "html.parser"),
    MovedModule("http_client", "httplib", "http.client"),
    MovedModule("email_mime_multipart", "email.MIMEMultipart", "email.mime.multipart"),
    MovedModule("email_mime_nonmultipart", "email.MIMENonMultipart", "email.mime.nonmultipart"),
    MovedModule("email_mime_text", "email.MIMEText", "email.mime.text"),
    MovedModule("email_mime_base", "email.MIMEBase", "email.mime.base"),
    MovedModule("BaseHTTPServer", "BaseHTTPServer", "http.server"),
    MovedModule("CGIHTTPServer", "CGIHTTPServer", "http.server"),
    MovedModule("SimpleHTTPServer", "SimpleHTTPServer", "http.server"),
    MovedModule("cPickle", "cPickle", "pickle"),
    MovedModule("queue", "Queue"),
    MovedModule("reprlib", "repr"),
    MovedModule("socketserver", "SocketServer"),
    MovedModule("_thread", "thread", "_thread"),
    MovedModule("tkinter", "Tkinter"),
    MovedModule("tkinter_dialog", "Dialog", "tkinter.dialog"),
    MovedModule("tkinter_filedialog", "FileDialog", "tkinter.filedialog"),
    MovedModule("tkinter_scrolledtext", "ScrolledText", "tkinter.scrolledtext"),
    MovedModule("tkinter_simpledialog", "SimpleDialog", "tkinter.simpledialog"),
    MovedModule("tkinter_tix", "Tix", "tkinter.tix"),
    MovedModule("tkinter_ttk", "ttk", "tkinter.ttk"),
    MovedModule("tkinter_constants", "Tkconstants", "tkinter.constants"),
    MovedModule("tkinter_dnd", "Tkdnd", "tkinter.dnd"),
    MovedModule("tkinter_colorchooser", "tkColorChooser",
                "tkinter.colorchooser"),
    MovedModule("tkinter_commondialog", "tkCommonDialog",
                "tkinter.commondialog"),
    MovedModule("tkinter_tkfiledialog", "tkFileDialog", "tkinter.filedialog"),
    MovedModule("tkinter_font", "tkFont", "tkinter.font"),
    MovedModule("tkinter_messagebox", "tkMessageBox", "tkinter.messagebox"),
    MovedModule("tkinter_tksimpledialog", "tkSimpleDialog",
                "tkinter.simpledialog"),
    MovedModule("urllib_parse", __name__ + ".moves.urllib_parse", "urllib.parse"),
    MovedModule("urllib_error", __name__ + ".moves.urllib_error", "urllib.error"),
    MovedModule("urllib", __name__ + ".moves.urllib", __name__ + ".moves.urllib"),
    MovedModule("urllib_robotparser", "robotparser", "urllib.robotparser"),
    MovedModule("xmlrpc_client", "xmlrpclib", "xmlrpc.client"),
    MovedModule("xmlrpc_server", "SimpleXMLRPCServer", "xmlrpc.server"),
    MovedModule("winreg", "_winreg"),
]
for attr in _moved_attributes:
    setattr(_MovedItems, attr.name, attr)
    if isinstance(attr, MovedModule):
        _importer._add_module(attr, "moves." + attr.name)
del attr

_MovedItems._moved_attributes = _moved_attributes

moves = _MovedItems(__name__ + ".moves")
_importer._add_module(moves, "moves")


class Module_six_moves_urllib_parse(_LazyModule):
    """Lazy loading of moved objects in six.moves.urllib_parse"""


_urllib_parse_moved_attributes = [
    MovedAttribute("ParseResult", "urlparse", "urllib.parse"),
    MovedAttribute("SplitResult", "urlparse", "urllib.parse"),
    MovedAttribute("parse_qs", "urlparse", "urllib.parse"),
    MovedAttribute("parse_qsl", "urlparse", "urllib.parse"),
    MovedAttribute("urldefrag", "urlparse", "urllib.parse"),
    MovedAttribute("urljoin", "urlparse", "urllib.parse"),
    MovedAttribute("urlparse", "urlparse", "urllib.parse"),
    MovedAttribute("urlsplit", "urlparse", "urllib.parse"),
    MovedAttribute("urlunparse", "urlparse", "urllib.parse"),
    MovedAttribute("urlunsplit", "urlparse", "urllib.parse"),
    MovedAttribute("quote", "urllib", "urllib.parse"),
    MovedAttribute("quote_plus", "urllib", "urllib.parse"),
    MovedAttribute("unquote", "urllib", "urllib.parse"),
    MovedAttribute("unquote_plus", "urllib", "urllib.parse"),
    MovedAttribute("urlencode", "urllib", "urllib.parse"),
    MovedAttribute("splitquery", "urllib", "urllib.parse"),
    MovedAttribute("splittag", "urllib", "urllib.parse"),
    MovedAttribute("splituser", "urllib", "urllib.parse"),
    MovedAttribute("uses_fragment", "urlparse", "urllib.parse"),
    MovedAttribute("uses_netloc", "urlparse", "urllib.parse"),
    MovedAttribute("uses_params", "urlparse", "urllib.parse"),
    MovedAttribute("uses_query", "urlparse", "urllib.parse"),
    MovedAttribute("uses_relative", "urlparse", "urllib.parse"),
]
for attr in _urllib_parse_moved_attributes:
    setattr(Module_six_moves_urllib_parse, attr.name, attr)
del attr

Module_six_moves_urllib_parse._moved_attributes = _urllib_parse_moved_attributes

_importer._add_module(Module_six_moves_urllib_parse(__name__ + ".moves.urllib_parse"),
                      "moves.urllib_parse", "moves.urllib.parse")


class Module_six_moves_urllib_error(_LazyModule):
    """Lazy loading of moved objects in six.moves.urllib_error"""


_urllib_error_moved_attributes = [
    MovedAttribute("URLError", "urllib2", "urllib.error"),
    MovedAttribute("HTTPError", "urllib2", "urllib.error"),
    MovedAttribute("ContentTooShortError", "urllib", "urllib.error"),
]
for attr in _urllib_error_moved_attributes:
    setattr(Module_six_moves_urllib_error, attr.name, attr)
del attr

Module_six_moves_urllib_error._moved_attributes = _urllib_error_moved_attributes

_importer._add_module(Module_six_moves_urllib_error(__name__ + ".moves.urllib.error"),
                      "moves.urllib_error", "moves.urllib.error")


class Module_six_moves_urllib_request(_LazyModule):
    """Lazy loading of moved objects in six.moves.urllib_request"""


_urllib_request_moved_attributes = [
    MovedAttribute("urlopen", "urllib2", "urllib.request"),
    MovedAttribute("install_opener", "urllib2", "urllib.request"),
    MovedAttribute("build_opener", "urllib2", "urllib.request"),
    MovedAttribute("pathname2url", "urllib", "urllib.request"),
    MovedAttribute("url2pathname", "urllib", "urllib.request"),
    MovedAttribute("getproxies", "urllib", "urllib.request"),
    MovedAttribute("Request", "urllib2", "urllib.request"),
    MovedAttribute("OpenerDirector", "urllib2", "urllib.request"),
    MovedAttribute("HTTPDefaultErrorHandler", "urllib2", "urllib.request"),
    MovedAttribute("HTTPRedirectHandler", "urllib2", "urllib.request"),
    MovedAttribute("HTTPCookieProcessor", "urllib2", "urllib.request"),
    MovedAttribute("ProxyHandler", "urllib2", "urllib.request"),
    MovedAttribute("BaseHandler", "urllib2", "urllib.request"),
    MovedAttribute("HTTPPasswordMgr", "urllib2", "urllib.request"),
    MovedAttribute("HTTPPasswordMgrWithDefaultRealm", "urllib2", "urllib.request"),
    MovedAttribute("AbstractBasicAuthHandler", "urllib2", "urllib.request"),
    MovedAttribute("HTTPBasicAuthHandler", "urllib2", "urllib.request"),
    MovedAttribute("ProxyBasicAuthHandler", "urllib2", "urllib.request"),
    MovedAttribute("AbstractDigestAuthHandler", "urllib2", "urllib.request"),
    MovedAttribute("HTTPDigestAuthHandler", "urllib2", "urllib.request"),
    MovedAttribute("ProxyDigestAuthHandler", "urllib2", "urllib.request"),
    MovedAttribute("HTTPHandler", "urllib2", "urllib.request"),
    MovedAttribute("HTTPSHandler", "urllib2", "urllib.request"),
    MovedAttribute("FileHandler", "urllib2", "urllib.request"),
    MovedAttribute("FTPHandler", "urllib2", "urllib.request"),
    MovedAttribute("CacheFTPHandler", "urllib2", "urllib.request"),
    MovedAttribute("UnknownHandler", "urllib2", "urllib.request"),
    MovedAttribute("HTTPErrorProcessor", "urllib2", "urllib.request"),
    MovedAttribute("urlretrieve", "urllib", "urllib.request"),
    MovedAttribute("urlcleanup", "urllib", "urllib.request"),
    MovedAttribute("URLopener", "urllib", "urllib.request"),
    MovedAttribute("FancyURLopener", "urllib", "urllib.request"),
    MovedAttribute("proxy_bypass", "urllib", "urllib.request"),
]
for attr in _urllib_request_moved_attributes:
    setattr(Module_six_moves_urllib_request, attr.name, attr)
del attr

Module_six_moves_urllib_request._moved_attributes = _urllib_request_moved_attributes

_importer._add_module(Module_six_moves_urllib_request(__name__ + ".moves.urllib.request"),
                      "moves.urllib_request", "moves.urllib.request")


class Module_six_moves_urllib_response(_LazyModule):
    """Lazy loading of moved objects in six.moves.urllib_response"""


_urllib_response_moved_attributes = [
    MovedAttribute("addbase", "urllib", "urllib.response"),
    MovedAttribute("addclosehook", "urllib", "urllib.response"),
    MovedAttribute("addinfo", "urllib", "urllib.response"),
    MovedAttribute("addinfourl", "urllib", "urllib.response"),
]
for attr in _urllib_response_moved_attributes:
    setattr(Module_six_moves_urllib_response, attr.name, attr)
del attr

Module_six_moves_urllib_response._moved_attributes = _urllib_response_moved_attributes

_importer._add_module(Module_six_moves_urllib_response(__name__ + ".moves.urllib.response"),
                      "moves.urllib_response", "moves.urllib.response")


class Module_six_moves_urllib_robotparser(_LazyModule):
    """Lazy loading of moved objects in six.moves.urllib_robotparser"""


_urllib_robotparser_moved_attributes = [
    MovedAttribute("RobotFileParser", "robotparser", "urllib.robotparser"),
]
for attr in _urllib_robotparser_moved_attributes:
    setattr(Module_six_moves_urllib_robotparser, attr.name, attr)
del attr

Module_six_moves_urllib_robotparser._moved_attributes = _urllib_robotparser_moved_attributes

_importer._add_module(Module_six_moves_urllib_robotparser(__name__ + ".moves.urllib.robotparser"),
                      "moves.urllib_robotparser", "moves.urllib.robotparser")


class Module_six_moves_urllib(types.ModuleType):
    """Create a six.moves.urllib namespace that resembles the Python 3 namespace"""
    __path__ = []  # mark as package
    parse = _importer._get_module("moves.urllib_parse")
    error = _importer._get_module("moves.urllib_error")
    request = _importer._get_module("moves.urllib_request")
    response = _importer._get_module("moves.urllib_response")
    robotparser = _importer._get_module("moves.urllib_robotparser")

    def __dir__(self):
        return ['parse', 'error', 'request', 'response', 'robotparser']

_importer._add_module(Module_six_moves_urllib(__name__ + ".moves.urllib"),
                      "moves.urllib")


def add_move(move):
    """Add an item to six.moves."""
    setattr(_MovedItems, move.name, move)


def remove_move(name):
    """Remove item from six.moves."""
    try:
        delattr(_MovedItems, name)
    except AttributeError:
        try:
            del moves.__dict__[name]
        except KeyError:
            raise AttributeError("no such move, %r" % (name,))


if PY3:
    _meth_func = "__func__"
    _meth_self = "__self__"

    _func_closure = "__closure__"
    _func_code = "__code__"
    _func_defaults = "__defaults__"
    _func_globals = "__globals__"
else:
    _meth_func = "im_func"
    _meth_self = "im_self"

    _func_closure = "func_closure"
    _func_code = "func_code"
    _func_defaults = "func_defaults"
    _func_globals = "func_globals"


try:
    advance_iterator = next
except NameError:
    def advance_iterator(it):
        return it.next()
next = advance_iterator


try:
    callable = callable
except NameError:
    def callable(obj):
        return any("__call__" in klass.__dict__ for klass in type(obj).__mro__)


if PY3:
    def get_unbound_function(unbound):
        return unbound

    create_bound_method = types.MethodType

    Iterator = object
else:
    def get_unbound_function(unbound):
        return unbound.im_func

    def create_bound_method(func, obj):
        return types.MethodType(func, obj, obj.__class__)

    class Iterator(object):

        def next(self):
            return type(self).__next__(self)

    callable = callable
_add_doc(get_unbound_function,
         """Get the function out of a possibly unbound function""")


get_method_function = operator.attrgetter(_meth_func)
get_method_self = operator.attrgetter(_meth_self)
get_function_closure = operator.attrgetter(_func_closure)
get_function_code = operator.attrgetter(_func_code)
get_function_defaults = operator.attrgetter(_func_defaults)
get_function_globals = operator.attrgetter(_func_globals)


if PY3:
    def iterkeys(d, **kw):
        return iter(d.keys(**kw))

    def itervalues(d, **kw):
        return iter(d.values(**kw))

    def iteritems(d, **kw):
        return iter(d.items(**kw))

    def iterlists(d, **kw):
        return iter(d.lists(**kw))
else:
    def iterkeys(d, **kw):
        return iter(d.iterkeys(**kw))

    def itervalues(d, **kw):
        return iter(d.itervalues(**kw))

    def iteritems(d, **kw):
        return iter(d.iteritems(**kw))

    def iterlists(d, **kw):
        return iter(d.iterlists(**kw))

_add_doc(iterkeys, "Return an iterator over the keys of a dictionary.")
_add_doc(itervalues, "Return an iterator over the values of a dictionary.")
_add_doc(iteritems,
         "Return an iterator over the (key, value) pairs of a dictionary.")
_add_doc(iterlists,
         "Return an iterator over the (key, [values]) pairs of a dictionary.")


if PY3:
    def b(s):
        return s.encode("latin-1")
    def u(s):
        return s
    unichr = chr
    if sys.version_info[1] <= 1:
        def int2byte(i):
            return bytes((i,))
    else:
        # This is about 2x faster than the implementation above on 3.2+
        int2byte = operator.methodcaller("to_bytes", 1, "big")
    byte2int = operator.itemgetter(0)
    indexbytes = operator.getitem
    iterbytes = iter
    import io
    StringIO = io.StringIO
    BytesIO = io.BytesIO
else:
    def b(s):
        return s
    # Workaround for standalone backslash
    def u(s):
        return unicode(s.replace(r'\\', r'\\\\'), "unicode_escape")
    unichr = unichr
    int2byte = chr
    def byte2int(bs):
        return ord(bs[0])
    def indexbytes(buf, i):
        return ord(buf[i])
    def iterbytes(buf):
        return (ord(byte) for byte in buf)
    import StringIO
    StringIO = BytesIO = StringIO.StringIO
_add_doc(b, """Byte literal""")
_add_doc(u, """Text literal""")


if PY3:
    exec_ = getattr(moves.builtins, "exec")


    def reraise(tp, value, tb=None):
        if value is None:
            value = tp()
        if value.__traceback__ is not tb:
            raise value.with_traceback(tb)
        raise value

else:
    def exec_(_code_, _globs_=None, _locs_=None):
        """Execute code in a namespace."""
        if _globs_ is None:
            frame = sys._getframe(1)
            _globs_ = frame.f_globals
            if _locs_ is None:
                _locs_ = frame.f_locals
            del frame
        elif _locs_ is None:
            _locs_ = _globs_
        exec("""exec _code_ in _globs_, _locs_""")


    exec_("""def reraise(tp, value, tb=None):
    raise tp, value, tb
""")


print_ = getattr(moves.builtins, "print", None)
if print_ is None:
    def print_(*args, **kwargs):
        """The new-style print function for Python 2.4 and 2.5."""
        fp = kwargs.pop("file", sys.stdout)
        if fp is None:
            return
        def write(data):
            if not isinstance(data, basestring):
                data = str(data)
            # If the file has an encoding, encode unicode with it.
            if (isinstance(fp, file) and
                isinstance(data, unicode) and
                fp.encoding is not None):
                errors = getattr(fp, "errors", None)
                if errors is None:
                    errors = "strict"
                data = data.encode(fp.encoding, errors)
            fp.write(data)
        want_unicode = False
        sep = kwargs.pop("sep", None)
        if sep is not None:
            if isinstance(sep, unicode):
                want_unicode = True
            elif not isinstance(sep, str):
                raise TypeError("sep must be None or a string")
        end = kwargs.pop("end", None)
        if end is not None:
            if isinstance(end, unicode):
                want_unicode = True
            elif not isinstance(end, str):
                raise TypeError("end must be None or a string")
        if kwargs:
            raise TypeError("invalid keyword arguments to print()")
        if not want_unicode:
            for arg in args:
                if isinstance(arg, unicode):
                    want_unicode = True
                    break
        if want_unicode:
            newline = unicode("\n")
            space = unicode(" ")
        else:
            newline = "\n"
            space = " "
        if sep is None:
            sep = space
        if end is None:
            end = newline
        for i, arg in enumerate(args):
            if i:
                write(sep)
            write(arg)
        write(end)

_add_doc(reraise, """Reraise an exception.""")

if sys.version_info[0:2] < (3, 4):
    def wraps(wrapped, assigned=functools.WRAPPER_ASSIGNMENTS,
              updated=functools.WRAPPER_UPDATES):
        def wrapper(f):
            f = functools.wraps(wrapped)(f)
            f.__wrapped__ = wrapped
            return f
        return wrapper
else:
    wraps = functools.wraps

def with_metaclass(meta, *bases):
    """Create a base class with a metaclass."""
    # This requires a bit of explanation: the basic idea is to make a dummy
    # metaclass for one level of class instantiation that replaces itself with
    # the actual metaclass.
    class metaclass(meta):
        def __new__(cls, name, this_bases, d):
            return meta(name, bases, d)
    return type.__new__(metaclass, 'temporary_class', (), {})


def add_metaclass(metaclass):
    """Class decorator for creating a class with a metaclass."""
    def wrapper(cls):
        orig_vars = cls.__dict__.copy()
        slots = orig_vars.get('__slots__')
        if slots is not None:
            if isinstance(slots, str):
                slots = [slots]
            for slots_var in slots:
                orig_vars.pop(slots_var)
        orig_vars.pop('__dict__', None)
        orig_vars.pop('__weakref__', None)
        return metaclass(cls.__name__, cls.__bases__, orig_vars)
    return wrapper

# Complete the moves implementation.
# This code is at the end of this module to speed up module loading.
# Turn this module into a package.
__path__ = []  # required for PEP 302 and PEP 451
__package__ = __name__  # see PEP 366 @ReservedAssignment
if globals().get("__spec__") is not None:
    __spec__.submodule_search_locations = []  # PEP 451 @UndefinedVariable
# Remove other six meta path importers, since they cause problems. This can
# happen if six is removed from sys.modules and then reloaded. (Setuptools does
# this for some reason.)
if sys.meta_path:
    for i, importer in enumerate(sys.meta_path):
        # Here's some real nastiness: Another "instance" of the six module might
        # be floating around. Therefore, we can't use isinstance() to check for
        # the six meta path importer, since the other six instance will have
        # inserted an importer with different class.
        if (type(importer).__name__ == "_SixMetaPathImporter" and
            importer.name == __name__):
            del sys.meta_path[i]
            break
    del i, importer
# Finally, add the importer to the meta path import hook.
sys.meta_path.append(_importer)



'''
 Licensed to the Apache Software Foundation (ASF) under one
 or more contributor license agreements.  See the NOTICE file
 distributed with this work for additional information
 regarding copyright ownership.  The ASF licenses this file
 to you under the Apache License, Version 2.0 (the
 "License"); you may not use this file except in compliance
 with the License.  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing,
 software distributed under the License is distributed on an
 "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
'''

import sys
import logging
import copy
import aliyunOpenApiData

LOG = logging.getLogger(__name__)


class Completer(object):

    def __init__(self):
        self.openApiDataHandler = aliyunOpenApiData.aliyunOpenApiDataHandler()
        self.driver = None
        self.main_hc = None
        self.main_options = ['output', 'AccessKeyId', 'AccessKeySecret', 'RegionId' ,'profile', 'version']
        self.cmdline = None
        self.point = None
        self.command_hc = None
        self.subcommand_hc = None
        self.command_name = None
        self.operation = None
        self.current_word = None
        self.previous_word = None
        self.non_options = None
        self.version = None
        self.aliyuncli = 'aliyuncli'

    def _complete_option(self, option_name):
        # if option_name == '--endpoint-url':
        #     return []
        if option_name == '--output':
            cli_data = ['text', 'table', 'json']
            return cli_data
        # if option_name == '--profile':
        #     return self.driver.session.available_profiles
        return []

    def _complete_provider(self):
        retval = []
        if self.current_word.startswith('-'):
            cw = self.current_word.lstrip('-')
            l = ['--' + n for n in self.main_options
                 if n.startswith(cw)]
            retval = l
        elif self.current_word == './testcli' or self.current_word == self.aliyuncli:
            retval = self._documented(self.openApiDataHandler.getApiCmdsLower())
        else:
            # Otherwise, see if they have entered a partial command name
            retval = self._documented(self.openApiDataHandler.getApiCmdsLower(),
                                      startswith=self.current_word)
        return retval

    def _complete_command(self):
        retval = []
        if self.current_word == self.command_name: # here means only cmd give operation is None
            _operations = set()
            apiOperations = self.openApiDataHandler.getApiOperations(self.command_name, self.version)
            import commandConfigure
            _configure = commandConfigure.commandConfigure()
            extensionOperations = _configure.getExtensionOperations(self.command_name)
            for item in apiOperations:
                _operations.add(item)
            if extensionOperations is not None:
                for item in extensionOperations:
                    _operations.add(item)
            if self.openApiDataHandler.getApiOperations(self.command_name, self.version):
                retval = self._documented(_operations)
            # retval = self._documented(self.openApiDataHandler.getApiOperations(self.command_name, self.version))
        elif self.current_word.startswith('-'): # this is complete the key and values
            retval = self._find_possible_options()
        else: # here means cmd give we need complete the operation
            # See if they have entered a partial command name
            _operations = set()
            apiOperations = self.openApiDataHandler.getApiOperations(self.command_name, self.version)
            import commandConfigure
            _configure = commandConfigure.commandConfigure()
            extensionOperations = _configure.getExtensionOperations(self.command_name)
            for item in apiOperations:
                _operations.add(item)
            if extensionOperations is not None:
                for item in extensionOperations:
                    _operations.add(item)
            if self.openApiDataHandler.getApiOperations(self.command_name, self.version):
                retval = self._documented(_operations, startswith=self.current_word)
                # retval = self._documented(self.openApiDataHandler.getApiOperations(self.command_name, self.version),
                #                           startswith=self.current_word)
        return retval

    def _documented(self, table, startswith=None):
        names = []
        for key in table:
            # if getattr(command, '_UNDOCUMENTED', False):
                # Don't tab complete undocumented commands/params
                # continue
            if startswith is not None and not key.startswith(startswith):
                continue
            # if getattr(command, 'positional_arg', False):
            #     continue
            names.append(key)
        return names

    def _complete_subcommand(self):
        retval = []
        if self.current_word == self.operation:
            retval = []
        elif self.current_word.startswith('-'):
            retval = self._find_possible_options()
        return retval

    def _find_possible_options(self):
        all_options = copy.copy(self.main_options)
        # here give all attribute list
        # where code run here , self.version should be decide before
        # self.subcommand_name = self.operation
        # cmdInstance = self.openApiDataHandler.getInstanceByCmd(self.command_name, self.operation, self.version)
        cmdInstance, mclassname = self.openApiDataHandler.getInstanceByCmdOperation(self.command_name, self.operation, self.version)
        # old_arg_list = self.openApiDataHandler.getAttrList(cmdInstance)
        old_arg_list = list()
        if cmdInstance is None:
            import commandConfigure
            _configure = commandConfigure.commandConfigure()
            old_arg_list = _configure.getExtensionOptions(self.command_name, self.operation)
        else:
            old_arg_list = self.openApiDataHandler.getAttrList(mclassname)
        new_arg_list = set()
        if not old_arg_list is None:
            for item in old_arg_list:
                if not item.startswith('_'):
                    new_arg_list.add(item)
            all_options = all_options + self._documented(new_arg_list)
        for opt in self.options:
            # Look thru list of options on cmdline. If there are
            # options that have already been specified and they are
            # not the current word, remove them from list of possibles.
            if opt != self.current_word:
                stripped_opt = opt.lstrip('-')
                if stripped_opt in all_options:
                    all_options.remove(stripped_opt)
        cw = self.current_word.lstrip('-')
        possibles = ['--' + n for n in all_options if n.startswith(cw)]
        if len(possibles) == 1 and possibles[0] == self.current_word:
            return self._complete_option(possibles[0])
        return possibles

    def _help_to_show_instance_attribute(self, classname):
        all_options = copy.copy(self.main_options)
        # here give all attribute list
        # where code run here , self.version should be decide before
        # self.subcommand_name = self.operation
        old_arg_list = self.openApiDataHandler.getAttrList(classname)
        new_arg_list = set()
        if not old_arg_list is None:
            for item in old_arg_list:
                if not item.startswith('_'):
                    new_arg_list.add(item)
            all_options = all_options + self._documented(new_arg_list)
        # for opt in self.options:
        #     # Look thru list of options on cmdline. If there are
        #     # options that have already been specified and they are
        #     # not the current word, remove them from list of possibles.
        #     if opt != self.current_word:
        #         stripped_opt = opt.lstrip('-')
        #         if stripped_opt in all_options:
        #             all_options.remove(stripped_opt)
        #cw = self.current_word.lstrip('-')
        possibles = ['--' + n for n in all_options]
        # if len(possibles) == 1 and possibles[0] == self.current_word:
        #     return self._complete_option(possibles[0])
        return possibles

    def _process_command_line(self):
        # Process the command line and try to find:
        #     - command_name
        #     - subcommand_name
        #     - words
        #     - current_word
        #     - previous_word
        #     - non_options
        #     - options
        self.command_name = None
        self.operation = None
        self.words = self.cmdline[0:self.point].split()
        self.current_word = self.words[-1]
        if len(self.words) >= 2:
            self.previous_word = self.words[-2]
        else:
            self.previous_word = None
        self.non_options = [w for w in self.words if not w.startswith('-')]
        self.options = [w for w in self.words if w.startswith('-')]
        # Look for a command name in the non_options
        for w in self.non_options:
            if w in self.openApiDataHandler.getApiCmdsLower() or w in self.openApiDataHandler.getApiCmds(): # cmd check
                self.command_name = w # here give the command_name
                if self.command_name.lower()=="oss":
                    return
                self.version = self.openApiDataHandler.getSdkVersion(self.command_name, None)
                cmd_obj = self.openApiDataHandler.getApiOperations(self.command_name, self.version)
                # self.command_hc = cmd_obj.create_help_command()
                if not cmd_obj is None:
                #     Look for subcommand name
                    for w in self.non_options:
                        if w in cmd_obj:
                            self.operation = w
                            # cmd_obj = self.command_hc.command_table[self.subcommand_name]
                            # self.subcommand_hc = cmd_obj.create_help_command()
                            break
                cmd_extension_obj = self.openApiDataHandler.getExtensionOperationsFromCmd(self.command_name)
                if not cmd_extension_obj is None:
                    for w in self.non_options:
                        if w in cmd_extension_obj:
                            self.operation = w
                            # cmd_obj = self.command_hc.command_table[self.subcommand_name]
                            # self.subcommand_hc = cmd_obj.create_help_command()
                            break
                break

    def complete(self, cmdline, point):
        self.cmdline = cmdline
        self.command_name = None
        if point is None:
            point = len(cmdline)
        self.point = point
        self._process_command_line()
        if not self.command_name: # such as 'ec'
            # If we didn't find any command names in the cmdline
            # lets try to complete provider options
            return self._complete_provider()
        if self.command_name and not self.operation: # such as 'ecs create-'
            return self._complete_command()
        return self._complete_subcommand()


def complete(cmdline, point):
    choices = Completer().complete(cmdline, point)
    print(' \n'.join(choices))


if __name__ == '__main__':
    # if len(sys.argv) == 3:
    #     cmdline = sys.argv[1]
    #     point = int(sys.argv[2])
    # elif len(sys.argv) == 2:
    #     cmdline = sys.argv[1]
    # else:
    #     print('usage: %s <cmdline> <point>' % sys.argv[0])
    #     sys.exit(1)
    cmdline = './testcli E'
    point = len(cmdline)
    print(complete(cmdline, point))



'''
 Licensed to the Apache Software Foundation (ASF) under one
 or more contributor license agreements.  See the NOTICE file
 distributed with this work for additional information
 regarding copyright ownership.  The ASF licenses this file
 to you under the Apache License, Version 2.0 (the
 "License"); you may not use this file except in compliance
 with the License.  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing,
 software distributed under the License is distributed on an
 "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
'''
#/usr/bin/env python
#-*- coding:utf-8 -*-

import os,sys
import aliyunCliParser
import platform


OSS_CREDS_FILENAME = "%s/.aliyuncli/osscredentials" % os.path.expanduser('~')
OSS_CONFIG_SECTION = 'OSSCredentials'

OAS_CREDS_FILENAME = "%s/.aliyuncli/oascredentials" % os.path.expanduser('~')
OAS_CONFIG_SECTION = 'OASCredentials'
def handleConfigure(cmd,operation):
    if not hasConfigureOperation(cmd,operation):
        handler = ConfigureCommand()
        handler._run_main()
    elif operation[0].lower() == 'list':
        handler = ConfigureListCommand()
        handler._run_main()
    elif operation[0].lower()== 'get':
        handler =ConfigureGetCommand()
        handler._run_main()
    elif operation[0].lower() == 'set':
        handler = ConfigureSetCommand()
        handler._run_main()


def hasConfigureOperation(cmd,operation):
    if  operation is not None:
        return True
    else :
        return False

class AliyunCredentials:
    aliyun_access_key_id = "aliyun_access_key_id"
    aliyun_access_key_secret = "aliyun_access_key_secret"

_WRITE_TO_CREDS_FILE = ['aliyun_access_key_id', 'aliyun_access_key_secret']
_WRITE_TO_CONFIG_FILE = ['region','output']




class AliyunConfig(object):
    def __init__(self,configWriter = None):
        if configWriter is None:
            configWriter = ConfigFileWriter()
        self._configWriter = configWriter
        self.home = ".aliyuncli"
        self.configure = "configure"
        self.credentials = "credentials"
        self.aliyunConfigurePath = os.path.join(self.findConfigureFilePath(),self.home)
        self.parser = aliyunCliParser.aliyunCliParser()

    def getConfig(self,profilename = None):
        if profilename is None:
            profilename = 'default'
        config = {}
        _configFileName = self.getConfigFileName()
        _credsFileName = self.getCredsFileName()
        self._getConfigFromFile(config,profilename,_configFileName)
        self._getConfigFromFile(config,profilename,_credsFileName)
        return config

    def _getConfigFromFile(self,config,profile,filename):
        if os.path.isfile(filename):
            with open(filename, 'r') as f:
                contents = f.readlines()
                if profile == 'default':
                    _sectionName = '%s'%profile
                else:
                    _sectionName = 'profile %s'%profile
                if self._configWriter.hasSectionName(_sectionName,contents)[0]:
                    sectionStart = self._configWriter.hasSectionName(_sectionName,contents)[1]
                    j = sectionStart
                    sectionEnd = self._configWriter._getSectionEnd(j,contents)
                    while j < sectionEnd:
                        line = contents[j]
                        if line.strip():
                            key = self._configWriter._getKey(line)
                            config[key] = self._configWriter._getValue(line)
                        j = j+1
                else:
                    pass
        else:
            pass

    def getConfigFileName(self):
        configFileName = os.path.join(self.aliyunConfigurePath,self.configure)
        return configFileName

    def getCredsFileName(self):
        credsFileName = os.path.join(self.aliyunConfigurePath,self.credentials)
        return credsFileName


    def findConfigureFilePath(self):
        homePath = ""
        if platform.system() == "Windows":
            homePath = os.environ['HOMEPATH']
            pass
        else:
            homePath = os.environ['HOME']
            pass
        return homePath


    def _getUserRegion(self):
        _keyValues = self.parser._getKeyValues()
        _profileName = _keyValues.get('--profile')
        if _profileName is not None and len(_profileName)>0:
            _profileName = _profileName[0]
        _config = self.getConfig(_profileName)
        _region = _config.get('region')
        return _region

    def _getUserFormat(self):
        _keyValues = self.parser._getKeyValues()
        _profileName = _keyValues.get('--profile')
        if _profileName is not None and len(_profileName)>0:
            _profileName = _profileName[0]
        _config = self.getConfig(_profileName)
        _format = _config.get('output')
        return _format


    def _getUserKey(self):
        _keyValues = self.parser._getKeyValues()
        _profileName = _keyValues.get('--profile')
        if _profileName is not None and len(_profileName)>0:
            _profileName = _profileName[0]
        _config = self.getConfig(_profileName)
        _userKey = _config.get(AliyunCredentials.aliyun_access_key_id)
        return _userKey

    def _getUserSecret(self):
        _keyValues = self.parser._getKeyValues()
        _profileName = _keyValues.get('--profile')
        if _profileName is not None and len(_profileName)>0:
            _profileName = _profileName[0]
        _config = self.getConfig(_profileName)
        _secretId = _config.get(AliyunCredentials.aliyun_access_key_secret)
        return _secretId

    def _getPort(self):
        _keyValues = self.parser._getKeyValues()
        _profileName = _keyValues.get('--profile')
        if _profileName is not None and len(_profileName)>0:
            _profileName = _profileName[0]
        _config = self.getConfig(_profileName)
        _port = _config.get('port')
        return _port

class InteractivePrompter(object):
    def __init__(self):
        pass
    def get_value(self, current_value, config_name, prompt_text=''):
        if config_name in ('aliyun_access_key_id', 'aliyun_access_key_secret'):
            if current_value != '' and not current_value is None:
                current_value = _mask_value(current_value)
        response = raw_input("%s [%s]: " % (prompt_text, current_value))
        if response is '':
            response = None
        return response

def _mask_value(current_value):
    if current_value is None:
        return 'None'
    else:
        if len(current_value) <4:
            count = 20 - len(current_value)
            return ('*'* count) + current_value[-4:]
        else:
            return ('*' * 16) + current_value[-4:]

def _createFile(filename):
    namePath = os.path.split(filename)[0]
    if not os.path.isdir(namePath):
        os.makedirs(namePath)
        with os.fdopen(os.open(filename,
                               os.O_WRONLY | os.O_CREAT, 0o600), 'w'):
            pass

class ConfigFileWriter(object):
    def __init__(self):
        pass

    def _updateConfig(self,new_values,config_filename):
        sectionName  = new_values.pop('__section__','default')
        if not os.path.isfile(config_filename):
            _createFile(config_filename)
            self._insertNewSection(sectionName,new_values,config_filename)
            return
        with open(config_filename, 'r') as f:
            contents = f.readlines()
        try:
            if self.hasSectionName(sectionName,contents)[0]:
                self._updateSectionContents(sectionName,contents, new_values)
                with open(config_filename, 'w') as f:
                    f.write(''.join(contents))
            else:
                self._insertNewSection(sectionName,new_values,config_filename)
        finally:
            f.close()



    def hasSectionName(self,sectionName,contents):
        result = False
        start = -1
        _sectionName = '['+sectionName+']'
        for i in range(len(contents)):
            line = contents[i]
            if line.strip():
                if line.strip().startswith(('#', ';')):
                    continue
                if line.strip().find(_sectionName)  >=0:
                    result = True
                    start = i
                    break
        return result,start

    def _updateSectionContents(self,sectionName,contents,new_values):
        new_values = new_values.copy()
        sectionStart = self.hasSectionName(sectionName,contents)[1]
        j = sectionStart
        sectionEnd = self._getSectionEnd(j,contents)
        while j < sectionEnd:
            line = contents[j]
            key = self._getKey(line)
            if key in new_values:
                new_value = new_values[key]
                new_line =  '%s = %s\n' % (key, new_value)
                contents[j] = new_line
                del new_values[key]
            j = j +1
        if new_values:
            if not contents[-1].endswith('\n'):
                contents.append('\n')
            self._insertNewValues(sectionEnd,contents,new_values)

    def _getSectionEnd(self,num,contents):
        k = len(contents)
        num = num +1
        while num <len(contents):
            line = contents[num]
            if line.strip():
                if line.strip().startswith(('#', ';')):
                    continue
                if  line.find('[') >=0 and line.find(']') :
                    return num
            num = num +1
        return k

    def _insertNewValues(self,num,contents,keyValues):
        new_contents = []
        for key, value in list(keyValues.items()):
            new_contents.append('%s = %s\n' % ( key, value))
            del keyValues[key]
        contents.insert(num, ''.join(new_contents))


    def _insertNewSection(self,sectionName,new_values,config_filename,num =0):
        with open(config_filename, 'a') as f:
            f.write('[%s]\n' % sectionName)
            contents = []
            self._insertNewValues(num ,contents,new_values)
            f.write(''.join(contents))


    def _getKey(self,line):
        key = None
        if line is not None and  line.strip().find('=') >0 :
            key = line.split("=",1)[0].strip()
        return key
    def _getValue(self,line):
        value = None
        if line is not None and  line.strip().find('=') >0 :
            value = line.split("=",1)[1].strip()
        return value
    def _getValueInSlice(self,start,end,key,contents):
        value = None
        while start <end:
            line = contents[start]
            start = start+1
            if key ==self._getKey(line):
                value = self._getValue(line)
                return value
        return value





class ConfigureSetCommand(object):
    def __init__(self,aliyunConfig= None,configWriter =None):
        if configWriter is None:
            configWriter = ConfigFileWriter()
        self._configWriter = configWriter
        if aliyunConfig is None:
            self.aliyunConfig = AliyunConfig()

    def _run_main(self):
        keyValues = self.aliyunConfig.parser._getKeyValues()
        _profilenameList = keyValues.get('--profile')
        _profilename = None
        if _profilenameList is not None:
            if len(_profilenameList) > 0:
                _profilename = _profilenameList[0]
        if _profilename is  not None:
            del keyValues['--profile']
        newValues = self.aliyunConfig.parser._getOpenApiKeyValues(keyValues)
        for key in newValues.keys():
            if newValues[key] is not None:
                if len(newValues[key]) > 0:
                    newValues[key] = newValues[key][0]
            else:
                del newValues[key]
        config_filename = self.aliyunConfig.getConfigFileName()
        if newValues:
            self._writeCredsToFile(newValues,_profilename)
            if _profilename is not None :
                newValues['__section__']=('profile %s' % _profilename)
            self._configWriter._updateConfig(newValues,config_filename)

    def _writeCredsToFile(self,new_values,profilename):
        credential_file_values = {}
        if 'AccessKeyId' in new_values:
            credential_file_values['aliyun_access_key_id'] = new_values.pop(
                'AccessKeyId')
        if 'AccessKeySecret' in new_values:
            credential_file_values['aliyun_access_key_secret'] = new_values.pop(
                'AccessKeySecret')
        creds_filename = self.aliyunConfig.getCredsFileName()
        if credential_file_values:
            if profilename is not None:
                credential_file_values['__section__'] = ('profile %s' % profilename)
            self._configWriter._updateConfig(credential_file_values,creds_filename)

class ConfigValue(object):

    def __init__(self, value, config_type, config_loc):
        self.value = value
        self.config_type = config_type
        self.config_loc = config_loc

    def mask_value(self):
        if self.value is None:
            return
        self.value = _mask_value(self.value)

class ConfigureListCommand(object):
    def __init__(self,aliyunConfig= None, stream=None,configWriter =None):
        if stream is None :
            self._stream = sys.stdout
        if aliyunConfig is None:
            self.aliyunConfig = AliyunConfig()
        if configWriter is None:
            configWriter = ConfigFileWriter()
        self._configWriter = configWriter
    def _run_main(self):
        self._displayConfigValue(ConfigValue('Value', 'Type', 'Location'),'Name')
        self._displayConfigValue(ConfigValue('-----', '----', '--------'),'----')
        keyValues = self.aliyunConfig.parser._getKeyValues()
        _profilenameList = keyValues.get('--profile')
        _profilename = None
        if _profilenameList is not None:
            if len(_profilenameList) > 0:
                _profilename = _profilenameList[0]
        sectionName = 'default'
        if _profilename is  not None:
            sectionName = ('profile %s' % _profilename)
        profile = ConfigValue(_profilename, 'None','None')
        self._displayConfigValue(profile,'Profile')
        config_filename = self.aliyunConfig.getConfigFileName()
        creds_filename = self.aliyunConfig.getCredsFileName()
        access_key, secret_key = self._lookup_credentials(sectionName,creds_filename)
        region,output = self._lookup_config(sectionName,config_filename)
        self._displayConfigValue(access_key,'Access_Key')
        self._displayConfigValue(secret_key,'Secret_Key')
        self._displayConfigValue(region,'Region')
        self._displayConfigValue(output,'Output')

    def _lookup_credentials(self,sectionName,filename):
        access_key = _WRITE_TO_CREDS_FILE[0]
        secret_key = _WRITE_TO_CREDS_FILE[1]
        access_type = None
        secret_type = None
        access_key_value = self.getConfigValue(access_key,sectionName,filename)
        secret_key_value = self.getConfigValue(secret_key,sectionName,filename)
        if access_key_value is not None:
            access_type = 'credentials'
        if secret_key_value is not None:
            secret_type = 'credentials'
        if access_key_value is None and secret_key_value is None:
            filename = None
        access_key_value = _mask_value(access_key_value)
        secret_key_value = _mask_value(secret_key_value)
        accessConfigValue =ConfigValue(access_key_value,access_type,filename)
        secretConfigValue =ConfigValue(secret_key_value,secret_type,filename)
        return accessConfigValue,secretConfigValue

    def _lookup_config(self,sectionName,filename):
        region_key = 'region'
        output_key = 'output'
        region_type = None
        output_type = None
        region_value = self.getConfigValue(region_key,sectionName,filename)
        output_value = self.getConfigValue(output_key,sectionName,filename)
        if region_value is not None:
            region_type ='configure'
        if output_value is not None:
            output_type = 'configure'
        if region_value is None and output_value is None:
            filename = None
        regionConfigValue = ConfigValue(region_value,region_type,filename)
        outputConfigValue = ConfigValue(output_value,output_type,filename)
        return regionConfigValue,outputConfigValue


    def getConfigValue(self,key,sectionName,filename):
        _value = None
        with open(filename, 'r') as f:
            contents = f.readlines()
        try:
            if self._configWriter.hasSectionName(sectionName,contents)[0]:
                j = self._configWriter.hasSectionName(sectionName,contents)[1]
                sectionEnd = self._configWriter._getSectionEnd(j,contents)
                while j < sectionEnd:
                    line = contents[j]
                    _key = self._configWriter._getKey(line)
                    if key == _key:
                        _value = self._configWriter._getValue(line)
                    j = j +1
        finally:
            f.close()
        return _value



    def _displayConfigValue(self,configValue,configName):
        self._stream.write('%10s %20s %15s    %20s\n' % (
            configName, configValue.value, configValue.config_type,
            configValue.config_loc))
        pass


class ConfigureCommand(object):
    def __init__(self,aliyunConfig= None,prompter =None,configWriter =None):
        if prompter is None:
            prompter = InteractivePrompter()
        self._prompter = prompter
        if configWriter is None:
            configWriter = ConfigFileWriter()
        self._configWriter = configWriter
        if aliyunConfig is None:
            self.aliyunConfig = AliyunConfig()

    name2prompt = [
        ('aliyun_access_key_id', "Aliyun Access Key ID"),
        ('aliyun_access_key_secret', "Aliyun Access Key Secret"),
        ('region', "Default Region Id"),
        ('output', "Default output format"),
    ]

    def _run_main(self):
        new_values = {}
        all_values = {}
        keyValues = self.aliyunConfig.parser._getKeyValues()
        _profilename = None
        _profilenameList = keyValues.get('--profile')
        if _profilenameList is not None:
            if len(_profilenameList) > 0:
                _profilename = _profilenameList[0]
        config = self.aliyunConfig.getConfig(_profilename)
        for name, prompt in self.name2prompt:
            value = config.get(name)
            new_value = self._prompter.get_value(value, name,prompt)
            all_values[name] = new_value
            if new_value is not None and new_value != value:
                new_values[name] = new_value
        config_filename = self.aliyunConfig.getConfigFileName()
        if all_values:
            self._writeCredsToOssFile(all_values)
            self._writeCredsToOasFile(all_values)
        if new_values:
            self._writeCredsToFile(new_values,_profilename)
            if _profilename is not None :
                new_values['__section__']=('profile %s' % _profilename)
            self._configWriter._updateConfig(new_values,config_filename)

    def _writeCredsToFile(self,new_values,profilename):
        credential_file_values = {}
        if 'aliyun_access_key_id' in new_values:
            credential_file_values['aliyun_access_key_id'] = new_values.pop(
                'aliyun_access_key_id')
        if 'aliyun_access_key_secret' in new_values:
            credential_file_values['aliyun_access_key_secret'] = new_values.pop(
                'aliyun_access_key_secret')
        creds_filename = self.aliyunConfig.getCredsFileName()
        if credential_file_values:
            if profilename is not None:
                credential_file_values['__section__'] = ('profile %s' % profilename)
            self._configWriter._updateConfig(credential_file_values,creds_filename)


    def _writeCredsToOssFile(self,new_values,profilename = OSS_CONFIG_SECTION):
        credential_file_values = {}
        if 'aliyun_access_key_id' in new_values:
            credential_file_values['accessid'] = new_values['aliyun_access_key_id']
        if 'aliyun_access_key_secret' in new_values:
            credential_file_values['accesskey'] = new_values['aliyun_access_key_secret']
        ossCredsFilename = OSS_CREDS_FILENAME
        if credential_file_values:
            if profilename is not None:
                credential_file_values['__section__'] = ('%s' % profilename)
            self._configWriter._updateConfig(credential_file_values,ossCredsFilename)

    def _writeCredsToOasFile(self,new_values,profilename = OAS_CONFIG_SECTION):
        credential_file_values = {}
        if 'aliyun_access_key_id' in new_values:
            credential_file_values['accessid'] = new_values['aliyun_access_key_id']
        if 'aliyun_access_key_secret' in new_values:
            credential_file_values['accesskey'] = new_values['aliyun_access_key_secret']
        oasCredsFilename = OAS_CREDS_FILENAME
        if credential_file_values:
            if profilename is not None:
                credential_file_values['__section__'] = ('%s' % profilename)
            self._configWriter._updateConfig(credential_file_values,oasCredsFilename)






class ConfigureGetCommand(object):
    def __init__(self,aliyunConfig= None,configWriter =None):
        if configWriter is None:
            configWriter = ConfigFileWriter()
        self._configWriter = configWriter
        if aliyunConfig is None:
            aliyunConfig = AliyunConfig()
        self.aliyunConfig = aliyunConfig

    def _run_main(self):
        keyValues = self.aliyunConfig.parser._getKeyValues()
        _profilenameList = keyValues.get('--profile')
        _profilename = None
        if _profilenameList is not None:
            if len(_profilenameList) > 0:
                _profilename = _profilenameList[0]
        operations = self.aliyunConfig.parser._getOperations()
        self._getKeysFromSection(_profilename,operations)

    def _getKeysFromSection(self,profilename,operations):
        if len(operations) >= 2:
            for i in range(1,len(operations)):
                operation = operations[i].strip()
                self._getKeyFromSection(profilename,operation)
        else:
            print 'The correct usage:aliyuncli configure get key --profile profilename'
            return

    def _getKeyFromSection(self,profilename,key):
        sectionName = 'default'
        if profilename is not None:
            sectionName = ('profile %s' % profilename)
        config_filename = self.aliyunConfig.getConfigFileName()
        creds_filename = self.aliyunConfig.getCredsFileName()
        if key in _WRITE_TO_CREDS_FILE :
            self._getKeyFromFile(creds_filename,sectionName,key)
        elif key in _WRITE_TO_CONFIG_FILE :
            self._getKeyFromFile(config_filename,sectionName,key)
        else:
            print key,'=','None'
    def _getKeyFromFile(self,filename,section,key):
        if  os.path.isfile(filename):
            with open(filename, 'r') as f:
                contents = f.readlines()
            if self._configWriter.hasSectionName(section,contents)[0]:
                start =  self._configWriter.hasSectionName(section,contents)[1]
                end = self._configWriter._getSectionEnd(start,contents)
                value = self._configWriter._getValueInSlice(start,end,key,contents)
                print key,'=',value
        else:
            print key,'=None'




if __name__ == "__main__":
    pass



'''
 Licensed to the Apache Software Foundation (ASF) under one
 or more contributor license agreements.  See the NOTICE file
 distributed with this work for additional information
 regarding copyright ownership.  The ASF licenses this file
 to you under the Apache License, Version 2.0 (the
 "License"); you may not use this file except in compliance
 with the License.  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing,
 software distributed under the License is distributed on an
 "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
'''
import re
import os,sys
import sys,imp,uuid
import aliyunExtensionCliHandler
import aliyunCliParser
import commandConfigure
import aliyunCliConfiugre
import aliyunSdkConfigure
import json
import cliError
import urllib2
import handleEndPoint

from __init__ import  __version__
_userAgent='aliyuncli/'+str(__version__)

version_cmds = ['ConfigVersion','ShowVersions']
END_POINT_OPERATION_LIST = ['ModifyEndPoint']

nonStandardSdkCmds = []


def oss_notice():
    print "OSS operation in aliyuncli is not supported."
    print "Please use 'ossutil' command line tool for Alibaba Cloud OSS operation."
    print "You can find information about 'ossutil' here: https://github.com/aliyun/ossutil.\n"

    
try:
    import oas
except Exception:
    pass
else:
    nonStandardSdkCmds.append('oas')
    import oasadp.oasHandler


def get_python_lib():
    ''''find aliyun sdk install path

    you will need install  corresponding  AliYun sdk for using aliyuncli subcommand . 
    For example, ECS sdk is needed for running  "$aliyuncli  ecs ", you can install the sdk with this cmd:  "$pip install aliyun-python-sdk-ecs"     

    >>> os.path.isdir(get_python_lib())
    True
    '''
    for path in sys.path:
        if path and os.path.isdir(path):
            objects=os.listdir(path)
            for object in objects:
                if object.startswith('aliyunsdk') and os.path.isdir(os.path.join(path,object)):
                    return path
    
    if len(sys.argv) >= 2 and sys.argv[1] == "oss":
        oss_notice()
        sys.exit(0)                        
    else:
        raise RuntimeError("aliyun sdk not properly installed, you will need install at least one sdk.\nECS sdk install cmd for example:  pip install aliyun-python-sdk-ecs ")            

class aliyunOpenApiDataHandler():
    def __init__(self, path=None):
        self.path = path
        self.extensionHandler = aliyunExtensionCliHandler.aliyunExtensionCliHandler()
        self.parser = aliyunCliParser.aliyunCliParser()

# this api will return all command from api, such as , ecs, rds, slb
    def getApiCmds(self):
        sitepackages_path=get_python_lib()
        cmds = list()
        cmds.extend(nonStandardSdkCmds)
        sub_objects=os.listdir(sitepackages_path)
        if sub_objects is not None:
            for object in sub_objects:
                if object.startswith('aliyunsdk') and os.path.isdir(os.path.join(sitepackages_path,object)):
                    cmd=object.split('aliyunsdk',1)[1]
                    if len(cmd)>0 and cmd not in['core']:
                        cmds.append(cmd)
        return set(cmds)

    def getApiCmdsLower(self):
        cmds = self.getApiCmds()
        lowerCmds = set()
        for cmd in cmds:
            lowerCmds.add(cmd)
        return lowerCmds

# this api will check if the cmd is the available
    def isAvailableCmd(self, cmdName):
        try:
            apiCmds = self.getApiCmds()
            for cmd in apiCmds:
                if cmdName.lower() == cmd.lower():
                    return True
            if cmdName == 'oss':  # just for displaying notification
                return True
            return False
        except Exception as e:
            return False
#this function is to handle no-POP SDK
    def isNonStandardSdkCmd(self,cmd):
        if cmd in nonStandardSdkCmds or cmd == 'oss':
            return True
        else:
            return False
#this function is to handle no-POP cmd
    def nonStandardSdkCmdHandle(self,cmd):
        if cmd == 'oss':
            oss_notice()

        if cmd == 'oas':
            self.handleOasCmd()
            
#this function is to handle no-POP oss
    def handleOasCmd(self):
        if len(sys.argv) >=3:
            oasadp.oasHandler.handleOas(sys.argv[2:])
        else:
            oasadp.oasHandler.handleOas()

# this api will define all operations from given command
    def getApiOperations(self, command,version):
        operations = []

        if command == 'oss':
            oss_notice()
            sys.exit(0)                        
        
        if command == 'oas':
            return oasadp.oasHandler.getAvailableOperations()

        sitepackages_path=get_python_lib()
        pre_module='aliyunsdk'
        module=pre_module+command
        sub_path='request'
        request_path=os.path.join(sitepackages_path,module,sub_path)
        version_path=os.path.join(request_path,str(version))
        for root, dirs, files in os.walk(version_path):
            for name in files:
                if name.endswith('Request.py'):
                    operation=name.split('Request.py',1)[0]
                    if len(operation) >0:
                        self.path=root
                        operations.append(operation)
        return set(operations)

    def getInstanceByCmdOperation(self,cmd,operation,version=None):
        if cmd is  None or operation is None:
            return None,None
        else:
            cmdInstance, mclassname = self.getInstance(operation,cmd,version)
            return cmdInstance, mclassname
# this api will return operations which is not comes from aliyun open api.
# parameter : cmd
# cmd will decide which operations need return.

    def getExtensionOperationsFromCmdLower(self, cmd):
        if cmd is None:
            return None
        defaultExtensionOpers=set(['configversion','showversions'])
        if cmd.lower() == "ecs":
            ecsExtensionOpers=set(['exportinstance', 'importinstance'])
            defaultExtensionOpers.update(ecsExtensionOpers)
        if cmd.lower() == "rds":
            rdsConfigure = commandConfigure.rds()
            rdsExtensionOpers = set()
            for item in rdsConfigure.extensionOperations:
                rdsExtensionOpers.add(item.lower())
            defaultExtensionOpers.update(rdsExtensionOpers)
        return defaultExtensionOpers

    def getExtensionOperationsFromCmd(self, cmd):
        if cmd is None :
            return None
        
        defaultExtensionOpers=set(['ConfigVersion','ShowVersions'])
        if cmd.lower() == "ecs":
            ecsConfigure = commandConfigure.ecs()
            cmdSet = set()
            for item in ecsConfigure.extensionOperations:
                cmdSet.add(item)
            defaultExtensionOpers.update(cmdSet)
        if cmd.lower() == "rds":
            rdsConfigure = commandConfigure.rds()
            cmdSet = set()
            for item in rdsConfigure.extensionOperations:
                cmdSet.add(item)
            defaultExtensionOpers.update(cmdSet)
        return defaultExtensionOpers
			
#check if the operation is ExtensionOperation
    def isAvailableExtensionOperation(self,cmd, operation):
        if operation is None:
            return False
        extension_operations = self.getExtensionOperationsFromCmdLower(cmd)
        if  extension_operations is None:
            return False
        elif operation.lower() in extension_operations:
            return True
        else :
            return False
# this api will check if the operation is the available
    def isAvailableOperation(self, cmd, operation, version=None):
        if operation is None:
            return False
        operations = self.getApiOperations(cmd,version)
        for item in operations:
            #if operation.lower() == item.lower():
            if operation == item:
                return True
        return False


# this method will give all attr of give class
    def getAttrList(self, classname):
        try:
            # here should be a instance
            SetFuncs=[]
            keys=[]
            if classname is not None:
                keys = classname.__dict__.keys()
            for key in keys:
                if key.startswith('set_'):
                    SetFuncs.append(key.replace('set_', ''))
            return SetFuncs
            # return classname.__dict__.keys()
        except Exception as e:

            return None
# this method will set all key:value for open api class
    def setAttr(self, classname, map):
        try:
            for key in map.keys():
                if len(map.get(key)) >= 1 and not key in ["AccessKeyId", "AccessKeySecret", "Endpoint"]:
                    value = map.get(key)[0]
                    key = key.replace("--","")
                    classname.__setattr__(key, value)
        except Exception as e:
            pass

# this method will change the endpoint for api command
    def changeEndPoint(self, classname, keyValues):
        endpoint = "Endpoint"
        try:
            if keyValues.has_key(endpoint) and keyValues[endpoint].__len__() > 0:
                classname._RestApi__domain = keyValues[endpoint][0]
        except Exception as e:
            pass

# this method will check if need set defaut region
    def needSetDefaultRegion(self, classname, map):
        need = True
        try:
            if len(map.get("RegionId")) >= 1: # user has set the regionId
                need = False
        except Exception as e:
            pass
        return need


# this method will create a instance by give class name
    def getInstance(self, operation,cmdName,version=None):
        if self.path is None:
            return None
        moduleName=operation+'Request'

        fp, pathname, desc = imp.find_module(moduleName,[self.path])
        imp.load_module(moduleName, fp, pathname, desc)
        modules_keys=sys.modules.keys()
        for key in modules_keys:
            if key==moduleName:
                module = sys.modules[moduleName]
                mInstance= getattr(module, moduleName)()
                className=getattr(module,moduleName)
                return mInstance,className
        raise Exception("Can not load module %s" % moduleName)

# the following api maybe need to remove
# this api will filter all numbers in one string
    def filterNumbers(self, _string):
        numbers = re.findall(r'(\w[0-9]+)\w*',_string) # this function will return a list
        if isinstance(numbers, list):
            if numbers.__len__() > 0 :
                numbers = ''.join(numbers)
                numbers = list(numbers)
                del numbers[0]
                numbers = ''.join(numbers)
        return numbers

# this api will return all
    def getFiles(self):
        rootPath = os.getcwdu() # here need to change
        files = list()
        apiPath = rootPath+self.path
        for parent, dirnames, filenames in os.walk(apiPath):
            for filename in filenames:
                if filename.endswith('.py'):
                    files.append(filename)
        return files

# this api will return all
    def getFilesByFilter(self, cmdAndVersion):
        rootPath = os.getcwdu() # here need to change
        files = list()
        apiPath = rootPath+self.path
        for parent, dirnames, filenames in os.walk(apiPath):
            for filename in filenames:
                if filename.endswith('.py') and filename.startswith(cmdAndVersion):
                    files.append(filename)
        return files

# this api will get ALL service version
    def getAllServiceVersion(self,cmd):
        pass

# this api will get ALL service latest version
    def getAllServiceLatestVersion(self):
        pass


    def hasNecessaryArgs(self,keyValues):
        region_id=self.getRegionId(keyValues)
        userKey=self.extensionHandler.getUserKey()
        userSecret=self.extensionHandler.getUserSecret()
        if region_id is None or userKey is None  or userSecret is None:
            return False
        else:
            return True

    def getResponse(self,cmd,operation,classname,instance,keyValues, secureRequest = False ):
        if secureRequest:
            instance.set_protocol_type('https')            
        
        setFuncs=self.getSetFuncs(classname)
        if len(setFuncs)>0:
            for func in setFuncs:
                key=func.split('set_',1)[1]
                if len(key)>0 and key in keyValues:
                    arg=keyValues[key]
                    if arg is not None and len(arg)>0:
                        param=arg[0]
                        try:
                            getattr(instance,func)(param)
                        except AttributeError as e:
                            # When the param is a json string, and it's handled as a 'repeat list',
                            # set it in the string format will raise AttributeError
                            # in this case, handle it in the json format
                            # FIXME 
                            if str(e) == "'str' object has no attribute 'get'":
                                try:
                                    param_in_obj = json.loads(param)
                                except ValueError:
                                    raise Exception("Can not handle param %s: not a valid json string" % key)
                                getattr(instance,func)(param_in_obj)
                            else:
                                raise e
        userKey=self.getUserKey()
        userSecret=self.getUserSecret()
        regionId=self.getRegionId(keyValues)
        userAgent=self.getUserAgent()
        port = self.getPort()
        module='aliyunsdkcore'
        try:
            from aliyunsdkcore import client
            from aliyunsdkcore.acs_exception.exceptions import ClientException  , ServerException
            Client=client.AcsClient(userKey,userSecret,regionId,True,3,userAgent,port)
            instance.set_accept_format('json')
            
            if hasattr(Client ,"do_action_with_exception"):
                result = Client.do_action_with_exception(instance)
            else:    
                result=Client.do_action(instance)
            jsonobj = json.loads(result)
            return jsonobj
        
        except ImportError as e:
            print module, 'is not exist!'
            sys.exit(1)            

        except ServerException as e:
            error = cliError.error()
            error.printInFormat(e.get_error_code(), e.get_error_msg())
            print "Detail of Server Exception:\n"
            print str(e)
            sys.exit(1)
        
        except ClientException as e:            
            # print e.get_error_msg()
            error = cliError.error()
            error.printInFormat(e.get_error_code(), e.get_error_msg())
            print "Detail of Client Exception:\n"
            print str(e)
            sys.exit(1)

    def getSetFuncs(self,classname):
        SetFuncs=[]
        keys=[]
        if classname is not None:
            keys = classname.__dict__.keys()
        for key in keys:
            if key.startswith('set_'):
                SetFuncs.append(key)
        return SetFuncs

# this api will return the special version from the configure file.
    def getVersionFromCfgFile(self, cmdName):
        pass

    def getAllVersionsByCmdName(self,command):
        versions=[]
        pre_module='aliyunsdk'
        module=pre_module+command
        sitepackages_path=get_python_lib()
        sub_path='request'
        module='aliyunsdk'+command
        request_path=os.path.join(sitepackages_path,module,sub_path)
        objects=os.listdir(request_path)
        for object in objects :
            if object.startswith('v') and os.path.isdir(os.path.join(request_path,object)):
                versions.append(object)
        versions.sort(reverse = True)
        return versions

    def getLatestVersion(self,versions):
        if versions is not None and len(versions)>0:
            return versions[0]

    def getTempVersion(self,keyValues):
        key='--version'
        if keyValues is not None and keyValues.has_key(key):
            return keyValues.get(key)
        key = 'version'
        if keyValues is not None and keyValues.has_key(key):
            return keyValues.get(key)

    def getVersionFromFile(self,cmd):
        version=None
        versionHandler=aliyunSdkConfigure.AliyunSdkConfigure()
        filename=versionHandler.fileName
        version=versionHandler.getCmdVersionFromFile(cmd,filename)
        return version

    def getSdkVersion(self,cmd,keyValues):
        tempVersion=self.getTempVersion(keyValues)
        versions=self.getAllVersionsByCmdName(cmd)
        if tempVersion is not None and len(tempVersion)>0:
            if tempVersion[0] in versions:
                return tempVersion[0]
            else:
                # show error
                error = cliError.error()
                error.printInFormat("Wrong version", "The version input is not exit.")
                return None
        configVersion=self.getVersionFromFile(cmd)
        if configVersion is not None:
            return configVersion
        defaultVersion=self.getLatestVersion(versions)
        if defaultVersion is not None:
            return defaultVersion
    def getUserKey(self):
        userKey=None
        userKey, userSecret = self.parser.getTempKeyAndSecret()
        if userKey is None:
            if self.extensionHandler.getUserKey() is not None:
                userKey = self.extensionHandler.getUserKey()
        return userKey

    def getUserSecret(self):
        userSecret=None
        userKey, userSecret = self.parser.getTempKeyAndSecret()
        if userSecret is None:
            if not self.extensionHandler.getUserSecret() is None:
                userSecret = self.extensionHandler.getUserSecret()
        return userSecret
    def getPort(self):
        port = self.extensionHandler.getPort()
        if port is None:
            port = 80
        return port
    def getRegionId(self,keyValues):
        key='RegionId'
        if key in keyValues:
            return keyValues[key][0]
        else:
            return None

    def getUserAgent(self):
        return _userAgent

    def getMacAddress(self):
        node = uuid.getnode()
        mac = uuid.UUID(int = node).hex[-12:]
        return mac

    def responseOptimize(self,response,cmd,operation):
        self.checkForServer(response,cmd,operation)
    def getRequestId(self,response):
        try:
            if response.has_key('RequestId') and len(response['RequestId']) > 0:
                requestId = response['RequestId']
                return  requestId
        except Exception:
            pass

    def checkForServer(self,response,cmd,operation):
        configure = aliyunCliConfiugre.configure()
        requestId = self.getRequestId(response)
        if requestId is None:
            requestId = ""
        ak =  self.getUserKey()
        if ak is None:
            ak = ""
        ua =  self.getUserAgent()
        if ua is None:
            ua = ""
        url = configure.server_url + "?requesId=" + requestId + "&ak=" + ak +"&ua="+ua+"&cmd="+cmd+"&operation="+operation
        try:
            f = urllib2.urlopen(url,data=None,timeout=5)
            s = f.read()
            return s
        except Exception :
            pass

    def isEndPointOperation(self,operation):
        if operation is not None and operation in END_POINT_OPERATION_LIST:
            return True
        else:
            return False

    def handleEndPointOperation(self,cmd,operation,keyValues):
        handleEndPoint.handleEndPoint(cmd,operation,keyValues)

if __name__ == '__main__':
    handler = aliyunOpenApiDataHandler()
    print "###############",handler.isAvailableExtensionOperation('ecs', 'exportInstance')
    print "###############",handler.isAvailableOperation('ecs', 'DescribeInstances')
    print "###############",handler.getExtensionOperationsFromCmd('ecs')



'''
 Licensed to the Apache Software Foundation (ASF) under one
 or more contributor license agreements.  See the NOTICE file
 distributed with this work for additional information
 regarding copyright ownership.  The ASF licenses this file
 to you under the Apache License, Version 2.0 (the
 "License"); you may not use this file except in compliance
 with the License.  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing,
 software distributed under the License is distributed on an
 "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
'''
#/usr/bin/env python
#!-*- coding:utf-8 -*-
import os
import urllib2
import cliError


def _paramOptimize(keyValues):
    if keyValues is None:
        return
    if not isinstance(keyValues,dict):
        return
    for key in keyValues.keys():
        if keyValues[key] is not None and len(keyValues[key]) >0:
            _getIndirectValues(keyValues[key])
        else:
            continue

def _getIndirectValues(values_list):
    if values_list is None:
        return
    if not isinstance(values_list,list):
        return
    count = len(values_list)
    for index in range(0,count):
        for prefix in PrefixMap:
            if values_list[index] is not None and isinstance(values_list[index],str):
                if values_list[index].startswith(prefix):
                    kwargs = KwargsMap.get(prefix, {})
                    data = PrefixMap[prefix](prefix,values_list[index], **kwargs)
                    values_list[index] =data

def _getParamFromFile(prefix,value,mode):
    path=value[len(prefix):]
    path = os.path.expanduser(path)
    path = os.path.os.path.expandvars(path)
    if not os.path.isfile(path):
        errorClass = cliError.error()
        errorMsg='file is not exist'
        errorClass.printInFormat(errorClass.CliException, errorMsg)
    try:
        with open(path, mode) as f:
            data = f.read()
            return data
    except (OSError, IOError) as e:
        print e
def _getParamFromUrl(prefix,value,mode):

    req = urllib2.Request(value)
    try:
        response=urllib2.urlopen(req)
        if response.getcode() == 200:
            return response.read()
        else:
            errorClass = cliError.error()
            errorMsg='Get the wrong content'
            errorClass.printInFormat(response.getcode(), errorMsg)
    except Exception as e:
        print e

PrefixMap = {'file://': _getParamFromFile,
             'fileb://': _getParamFromFile
             #'http://': _getParamFromUrl,
             #'https://': _getParamFromUrl
            }

KwargsMap = {'file://': {'mode': 'r'},
             'fileb://': {'mode': 'rb'},
             #'http://': {},
             #'https://': {}
            }


__author__ = 'zhaoyang.szy'

def handleEndPoint(cmd,operation,keyValues):
    if not hasNecessaryArgs(keyValues):
        print 'RegionId/EndPoint is absence'
        return
    if cmd is not None:
        cmd = cmd.capitalize()
    regionId = getRegionId(keyValues)
    endPoint = getEndPoint(keyValues)
    _handleEndPoint(cmd,regionId,endPoint)


def hasNecessaryArgs(keyValues):
    if keyValues is not None and isinstance(keyValues,dict):
        if keyValues.get('RegionId') is not None and keyValues.get('EndPoint') is not None:
            return True
    else:
        return False



def _handleEndPoint(cmd,regionId,endPoint):
    try:
        from aliyunsdkcore.profile.region_provider import modify_point
        modify_point(cmd,regionId,endPoint)
    except Exception as e:
        print e
        pass


def getRegionId(keyValues):
    if keyValues is not None and isinstance(keyValues,dict):
        regionId = keyValues.get('RegionId')
        if regionId is not None and isinstance(regionId,list) and len(regionId)>0:
            return regionId[0]

def getEndPoint(keyValues):
    if keyValues is not None and isinstance(keyValues,dict):
        endPoint = keyValues.get('EndPoint')
        if endPoint is not None and isinstance(endPoint,list) and len(endPoint)>0:
            return  endPoint[0]



import os,sys,json
parentdir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0,parentdir)
import aliyunExtensionCliHandler
import aliyunOpenApiData
import aliyunCliParser
import response 
class RdsImportDBInstanceHandler:
    def __init__(self):
        self.parser = aliyunCliParser.aliyunCliParser()
        self.apiHandler = aliyunOpenApiData.aliyunOpenApiDataHandler()
        self.extensionHandler = aliyunExtensionCliHandler.aliyunExtensionCliHandler()
    
    def getFileName(self,keyValues):
        filename = None
        if keyValues.has_key('--filename') and len(keyValues['--filename']) > 0:
            filename = keyValues['--filename'][0]
        else:
            return filename, "A filename is needed! please use \'--filename\' and add the file name."
        return filename, ""
    def getInstanceCount(self,keyValues):
        count = 1
        import_count = "--count"
        if keyValues.has_key(import_count) and len(keyValues[import_count]) > 0:
            if  keyValues[import_count][0].isdigit() and int(keyValues[import_count][0]) >= 0:
                count = keyValues[import_count][0]
            else:
                pass
                # print "InstanceCount should be a positive number! The default value(1) will be used!"
        return int(count), "InstanceCount is "+str(count)+" created."
    
    def getSubOperations(self,cmd,operation):
        import commandConfigure
        _rds = commandConfigure.rds()
        operations = None
        if cmd.lower() == 'rds' and operation.lower() ==_rds.importDBInstance.lower():
            operations = ['CreateDBInstance']
        return operations 

    def _handSubOperation(self,cmd,operations,keyValues,version,secureRequest=False):
        for item in operations:
            if self.apiHandler.isAvailableOperation(cmd, item, version):
                cmdInstance, mclassname = self.apiHandler.getInstanceByCmdOperation(cmd, item, version)
                if not cmdInstance is None:
                    newkeyValues = self.parser.getOpenApiKeyValues(keyValues)
                    if self.apiHandler.needSetDefaultRegion(cmdInstance, newkeyValues):
                        newkeyValues["RegionId"] = [self.extensionHandler.getUserRegion()]
                    newkeyValues["ClientToken"] = [self.random_str()]
                    # print newkeyValues.keys()
                    # return
                    # self._setAttr(cmdInstance, newkeyValues) # set all key values in instance
                    # self.apiHandler.changeEndPoint(cmdInstance, newkeyValues)
                    try:
                        result = self.apiHandler.getResponse(cmd, item, mclassname, cmdInstance, newkeyValues,secureRequest)
                        self.apiHandler.responseOptimize(result,cmd,item)
                        # result = cmdInstance.getResponse()
                        if("Code" in result):
                            response.display_response("error", result, "json")
                        else:
                            response.display_response(item, result, "json")
                    except Exception,e:
                        print(e)
   
    def getKVFromJson(self,filename):
        inputFilePath  = os.path.split(filename)[0]
        inputFileName = os.path.split(filename)[1]
        if not os.path.isfile(filename):
            if inputFilePath == '':
                filePath = self.extensionHandler.aliyunConfigurePath
            else:
                filePath = inputFilePath
            # fileName = filePath + inputFileName
            fileName = os.path.join(filePath, inputFileName)
        else:
            fileName = filename
        try:
            fp = open(fileName,'r')
            data=json.loads(fp.read())
            keys = data.keys()
            # print keys, type(data['Items']['DBInstanceAttribute'][0])
            # instanceAttribute = data['Items']['DBInstanceAttribute'][0]
            items = data['Items']['DBInstanceAttribute'][0]
            keys = items.keys()
            newMap = dict()
            for key in keys:
                value = list()
                value.append(items[key])
                newMap[key] = value
            return newMap
            # return data['Items']['DBInstanceAttribute'][0]
        finally:
            fp.close()

    def importInstance(self,cmd,operation,version,secureRequest = False):
        _keyValues = self.parser.getCliKeyValues()
        operations = self.getSubOperations(cmd,operation)
        _instanceCount, countMsg = self.getInstanceCount(_keyValues)
        filename, errorMsg = self.getFileName(_keyValues)
        if filename is None:
            import cliError
            errorClass = cliError.error()
            errorClass.printInFormat(errorClass.CliException, errorMsg)
            return
        keyValues = self.getKVFromJson(filename)
        keyValues['PayType'] = ["Postpaid"]
        for i in range(1,_instanceCount+1):
            self._handSubOperation(cmd,operations,keyValues,version,secureRequest)

    # this method will set all key:value for open api class
    def _setAttr(self, classname, map):
        try:
            for key in map.keys():
                if  not key in ["AccessKeyId", "AccessKeySecret", "Endpoint"]:
                    value = map.get(key)
                    classname.__setattr__(key, value)
        except Exception as e:
            pass

    def random_str(self, randomlength=30):
        from random import Random
        str = ''
        chars = 'AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz0123456789'
        length = len(chars) - 1
        random = Random()
        for i in range(randomlength):
            str += chars[random.randint(0, length)]
        return str
 

if __name__ == "__main__":
    handler = RdsImportDBInstanceHandler()
    # handler.getKVFromJson('ttt')
    # print handler.getKVFromJson('ttt')
    print handler.random_str()





'''
 Licensed to the Apache Software Foundation (ASF) under one
 or more contributor license agreements.  See the NOTICE file
 distributed with this work for additional information
 regarding copyright ownership.  The ASF licenses this file
 to you under the Apache License, Version 2.0 (the
 "License"); you may not use this file except in compliance
 with the License.  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing,
 software distributed under the License is distributed on an
 "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
'''

import aliyunOpenApiData
import cliError
class aliyunCliHelper:
    def __init__(self):
        self.openApiDataHandler = aliyunOpenApiData.aliyunOpenApiDataHandler()

    def showUsage(self):
        print "usage: aliyuncli <command> <operation> [options and parameters]"

    def showExample(self):
        print "show example"

    def showCmdError(self, cmd):
        self.showUsage()
        print "<aliyuncli> the valid command as follows:\n"
        cmds = self.openApiDataHandler.getApiCmds()
        self.printAsFormat(cmds)

    def showOperationError(self, cmd, operation):
        keyValues=self.openApiDataHandler.parser._getKeyValues()
        version = self.openApiDataHandler.getSdkVersion(cmd,keyValues)
        versions=self.openApiDataHandler.getAllVersionsByCmdName(cmd)
        if version not in versions:
                error = cliError.error()
                error.printInFormat("Wrong version", "The sdk version is not exit.")
                return None
        self.showUsage()
        print "["+cmd+"]","valid operations as follows:\n"
        operations = self.openApiDataHandler.getApiOperations(cmd, version)
        extensions = self.openApiDataHandler.getExtensionOperationsFromCmd(cmd)
        operations.update(extensions)
        import commandConfigure
        if cmd.lower() == 'rds':
            rdsConfigure = commandConfigure.rds()
            operations.add(rdsConfigure.exportDBInstance)
            operations.add(rdsConfigure.importDBInstance)
        self.printAsFormat(operations)

    def showParameterError(self, cmd, operation, parameterlist):
        print 'usage: aliyuncli <command> <operation> [options and parameters]'
        print '['+cmd+"."+operation+']: current operation can uses parameters as follow :\n'
        self.printAsFormat(parameterlist)
        pass

    def printAsFormat(self, data):
        mlist = list()
        for item in data:
            mlist.append(item)
        mlist.sort()
        count = 0
        tmpList = list()
        for item in mlist:
            tmpList.append(item)
            count = count+1
            if len(tmpList) == 2:
                print '{0:40}'.format(tmpList[0]),'\t|',format(tmpList[1],'<10')
                tmpList = list()
            if len(tmpList) == 1 and count == len(mlist):
                print tmpList[0]


'''
 Licensed to the Apache Software Foundation (ASF) under one
 or more contributor license agreements.  See the NOTICE file
 distributed with this work for additional information
 regarding copyright ownership.  The ASF licenses this file
 to you under the Apache License, Version 2.0 (the
 "License"); you may not use this file except in compliance
 with the License.  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing,
 software distributed under the License is distributed on an
 "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
'''
#!/usr/bin/env python
#-*- coding:utf-8 -*-
import os,platform,sys
import aliyunCliParser

def handleSdkVersion(cmd,operation,version):
    handler=AliyunSdkConfigure()
    if  operation.lower() == 'configversion':
        handler.sdkConfigure(cmd,operation)
    elif operation.lower()== 'showversions':
        handler.showVersions(cmd,operation)


class AliyunSdkConfigure(object):
    def __init__(self):
        self.home = ".aliyuncli"
        self.sdkFile = "sdk_version"
        self.aliyunConfigurePath = os.path.join(self.findConfigureFilePath(),self.home)
        self.fileName=os.path.join(self.aliyunConfigurePath,self.sdkFile)
        self.parser = aliyunCliParser.aliyunCliParser()

    def sdkConfigure(self,cmd,operation):
        keyValues = self.parser._getKeyValues()
        if keyValues.has_key('--version') and len(keyValues['--version']) > 0:
            version=keyValues['--version'][0]
            filename=self.fileName
            self.writeCmdVersionToFile(cmd,version,filename)
        else:
            print "A argument is needed! please use \'--version\' and add the sdk version."
            return
    def showVersions(self,cmd,operation,stream=None):
        configureVersion='(not configure)'
        if stream is None:
            stream=sys.stdout
        getVersion=self.getCmdVersionFromFile(cmd,self.fileName)
        if  getVersion is not None:
            configureVersion=getVersion
        import aliyunOpenApiData
        apiHandler =aliyunOpenApiData.aliyunOpenApiDataHandler()
        versions =apiHandler.getAllVersionsByCmdName(cmd)
        stream.write('%s%s\n' % ('* ',configureVersion))
        for version in versions:
            stream.write('  %s\n' % (version))




    def findConfigureFilePath(self):
        homePath = ""
        if platform.system() == "Windows":
            homePath = os.environ['HOMEPATH']
            pass
        else:
            homePath = os.environ['HOME']
            pass
        return homePath

    def getCmdVersionFromFile(self,cmd,filename):
        if not os.path.isfile(filename):
            return None
        with open(filename, 'r') as f:
            contents = f.readlines()
        try:
            if self.hasCmdName(cmd,contents):
                keyValues=self._getKeyValues(contents)
                version=keyValues.get(cmd)
                return version
            else:
                return None
        finally:
            f.close()

    def updateCmdVersion(self,cmd,version,filename):
        if not os.path.isfile(filename):
            self._createFile(filename)
            self.insertCmdVersion(cmd,version,filename)
            return
        with open(filename, 'r') as f:
            contents = f.readlines()
            try:
                if self.hasCmdName(cmd,contents):
                    self._updateContents(cmd,version,contents)
                    with open(filename, 'w') as f:
                        f.write(''.join(contents))
                else:
                    self.insertCmdVersion(cmd,version,filename)
            finally:
                f.close()

    def insertCmdVersion(self,cmd,version,filename):
        with open(filename, 'a') as f:
            contents = []
            self._insertNewCmdVersion(cmd,version,contents)
            f.write(''.join(contents))

    def writeCmdVersionToFile(self,cmd,version,filename):
        if cmd is not None and version is not None :
            if filename is not None :
                self.updateCmdVersion(cmd,version,filename)

    def hasCmdName(self,cmd,contents):
        keys=self._getKeys(contents)
        if cmd in keys:
            return True
        else:
            return False

    def _updateContents(self,cmd,version,contents):
        for j in range(len(contents)):
            line = contents[j]
            key = self._getKey(line)
            if cmd ==key:
                new_line = '%s=%s\n' %(cmd,version)
                contents[j] = new_line
                break

    def _insertNewCmdVersion(self,cmd,version,contents):
        new_contents = []
        new_contents.append('%s=%s\n' %(cmd,version))
        contents.append(''.join(new_contents))

    def _getKeys(self,contents):
        keys=[]
        for j in range(len(contents)):
            line = contents[j]
            key = self._getKey(line)
            keys.append(key)
        return keys

    def _getKey(self,line):
        key = None
        if line is not None and  line.strip().find('=') >0 :
            key = line.split("=",1)[0].strip()
        return key

    def _getKeyValues(self,contents):
        keyValues={}
        for j in range(len(contents)):
            line = contents[j]
            if line is not None and  line.strip().find('=') >0 :
                key = line.split("=",1)[0].strip()
                value = line.split("=",1)[1].strip()
                keyValues[key]=value
        return keyValues

    def _createFile(self,filename):
        namePath = os.path.split(filename)[0]
        if not os.path.isdir(namePath):
            os.makedirs(namePath)
            with os.fdopen(os.open(filename,
                                   os.O_WRONLY | os.O_CREAT, 0o600), 'w'):
                pass



'''
 Licensed to the Apache Software Foundation (ASF) under one
 or more contributor license agreements.  See the NOTICE file
 distributed with this work for additional information
 regarding copyright ownership.  The ASF licenses this file
 to you under the Apache License, Version 2.0 (the
 "License"); you may not use this file except in compliance
 with the License.  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing,
 software distributed under the License is distributed on an
 "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
'''
import sys,json
import paramOptimize

class aliyunCliParser():
    def __init__(self):
        self.args = sys.argv[1:]
        pass

# this function find cli cmd
    def getCliCmd(self):
        if self.args.__len__() >= 1:
            return self.args[0].lower()

# this function find cli operation
    def getCliOperation(self):
        if self.args.__len__() >= 2:
            return self.args[1]

    def _getCommand(self):
        if self.args.__len__() >=1:
            return self.args[0]

    def _getOperations(self):
        operations = []
        i =1
        _len = self.args.__len__()
        if _len >=2:
            while i < _len:
                if  self.args[i].strip().find('--'):
                    operations.append(self.args[i])
                else:
                    break
                i =i+1
        if len(operations):
            return operations
        else :
            return None



    def _getKeyValues(self):
        keyValues = dict()
        len = self.args.__len__()

        # check is using ecs cli now?
        ecsFlag = False
        if "ecs" == str(self.getCliCmd()):
            ecsFlag = True

        emrFlag = False            
        if  "emr"  ==  str(self.getCliCmd()):
            emrFlag = True

        if len >= 2:
            current=1
            while current <len:
                #values = list()
                if self.args[current].strip().startswith('--'):
                    key=self.args[current].strip()
                    if ecsFlag:  # if using ecs cli, we need compate xx.yy.zz and xxyyzz in cli
                        key = key.replace('.', '')
                    start=current + 1
                    values=list()
                    while start <len and not self.args[start].strip().startswith('--'):
                        v = self.args[start].strip()
                        if emrFlag and   key.strip("--") in {'BootstrapActions', 'ClusterTypeLists', 'Columns', 'EcsOrders', 'EcsResetAutoRenewDos', 'ExecutionPlanIdLists', 'JobIdLists', 'OptionSoftWareLists', 'RenewEcsDos', 'StatusLists'} :
                            try:
                                v  = v.replace("'","\"")
                                v = json.loads(v)
                            except ValueError as e:
                                print("aliyuncli JSON argument parse failed. Detail message:")
                                print("\tcurrent argument:{0}".format(key))
                                print("\tdiagnostic message: {0} ".format(e.message))
                                sys.exit(-1)
                        values.append(v)
                        start=start+1
                    keyValues[key] = values
                    current=start
                else:
                    current=current+1
        paramOptimize._paramOptimize(keyValues)
        return keyValues

# this function find cli key:values , notice here is values , we need consider multiple values case
# --args is key, and if no -- is value
    def getCliKeyValues(self):
        keyValues = dict()
        len = self.args.__len__()
        if len >= 3:
            left_index = 2
            if self.args[1].find("--") >= 0:
                left_index = 1
            for index in range(left_index, len):
                currentValue = self.args[index]
                if currentValue.find('--') >= 0 : # this is command
                    index = index+1 # check next args
                    values = list()
                    while index < len and self.args[index].find('--') < 0:
                        values.append(self.args[index])
                        index = index + 1
                    keyValues[currentValue] = values
        return keyValues

# this function will find the temp key and secret if user input the --key and --value
    def getTempKeyAndSecret(self):
        keyValues = dict()
        len = self.args.__len__()
        keystr = "--AccessKeyId"
        secretstr = "--AccessKeySecret"
        _key = None
        _secret = None
        if len >= 3:
            for index in range(2, len):
                currentValue = self.args[index]
                if currentValue.find('--') >= 0 : # this is command
                    index = index+1 # check next args
                    values = list()
                    while index < len and self.args[index].find('--') < 0:
                        values.append(self.args[index])
                        index = index + 1
                    keyValues[currentValue] = values
        if keyValues.has_key(keystr) and keyValues[keystr].__len__() > 0:
            _key = keyValues[keystr][0]
        if keyValues.has_key(secretstr) and keyValues[secretstr].__len__() > 0:
            _secret = keyValues[secretstr][0]
        #print "accesskeyid: ", _key , "accesskeysecret: ",_secret
        return _key, _secret



# this function will give all extension command defined by us
    def getAllExtensionCommands(self):
        cmds = list()
        cmds = ['help', '-h', '--help', ]
        return cmds

# this function will filter all key and values which is in openApi
    def getOpenApiKeyValues(self, map):
        keys = map.keys()
        newMap = dict()
        for key in keys:
            value = map.get(key)
            key = key.replace('--', '')
            newMap[key] = value
        return newMap
    

    def _getOpenApiKeyValues(self, map):
        keys = map.keys()
        newMap = dict()
        for key in keys:
            value = map.get(key)
            key = key.replace('--', '')
            newMap[key] = value
        return newMap

# this function will filter all key and values which is in extension command

# this function will filter all key and values which is in extension command
    def getExtensionKeyValues(self, map):
        pass

# this function will return output format from key values
    def getOutPutFormat(self, map):
        keys = map.keys()
        for key in keys:
            if key == '--output' :
                return map.get(key)
        return None

# this function will return whether to use HTTPS request
    def getSecureChoice(self, map):
        keys = map.keys()
        for key in keys:
            if key == '--secure' :
                return True 
        return False
    





import os,sys,json
parentdir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0,parentdir)
import aliyunExtensionCliHandler
import aliyunOpenApiData
import aliyunCliParser
import response 
class EcsExportHandler:
    def __init__(self):
        self.parser = aliyunCliParser.aliyunCliParser()
        self.apiHandler = aliyunOpenApiData.aliyunOpenApiDataHandler()
        self.extensionHandler = aliyunExtensionCliHandler.aliyunExtensionCliHandler()
    
    def getFileName(self,keyValues):
        filename = None
        if keyValues.has_key('--filename') and len(keyValues['--filename']) > 0:
            filename = keyValues['--filename'][0]
        else:
            return filename, "A file name is needed! please use \'--filename\' and add the file name."
        return filename, ''
    
    def exportInstance(self,cmd,operation,version,secureRequest = False):
        if cmd.lower() == 'ecs' and operation.lower() =='exportinstance':
            operations = ['DescribeInstanceAttribute']
        else :
            return None
        _keyValues = self.parser.getCliKeyValues()
        # filename = self.getFileName(_keyValues)
        filename, errorMsg = self.getFileName(_keyValues)
        if filename is None:
            import cliError
            errorClass = cliError.error()
            errorClass.printInFormat(errorClass.CliException, errorMsg)
            return
        for item in operations:
            if self.apiHandler.isAvailableOperation(cmd, item, version):
                cmdInstance, mclassname = self.apiHandler.getInstanceByCmdOperation(cmd, item, version)
                if not cmdInstance is None:
                    newkeyValues = self.parser.getOpenApiKeyValues(_keyValues)
                    if self.apiHandler.needSetDefaultRegion(cmdInstance, newkeyValues):
                        newkeyValues["RegionId"] = [self.extensionHandler.getUserRegion()]
                    self.apiHandler.setAttr(cmdInstance, newkeyValues) # set all key values in instance
                    self.apiHandler.changeEndPoint(cmdInstance, newkeyValues)
                    try:
                        #result = cmdInstance.getResponse()
                        result = self.apiHandler.getResponse(cmd,operation, mclassname, cmdInstance, newkeyValues,secureRequest)
                        result = self._optimizeResult(result)
                        self.apiHandler.responseOptimize(result,cmd,operation)
                        if("Code" in result):
                            response.display_response("error", result, "json")
                        else:
                            if not filename == None:
                                self.exportInstanceToFile(result,filename)
                            else:
                                print 'Filename is needed'
                    except Exception,e:
                        print(e)
    def _optimizeResult(self,result):
        keys = result.keys()
        if 'SecurityGroupIds' in keys and 'SecurityGroupId' in result['SecurityGroupIds'] :
            SGIds = result['SecurityGroupIds']['SecurityGroupId'][0]    
            result['SecurityGroupId'] = SGIds
        rubbishKeys = ('SecurityGroupIds','CreationTime','PublicIpAddress','VpcAttributes','Status',\
'EipAddress','SerialNumber','OperationLocks','RequestId','InnerIpAddress',)
        for item in keys:
            if item in rubbishKeys:
                del result[item]
        return result
    def exportInstanceToFile(self,result,filename):
        inputFilePath  = os.path.split(filename)[0]
        inputFileName = os.path.split(filename)[1]
        if not os.path.isfile(filename):
            if inputFilePath == '':
                filePath = self.extensionHandler.aliyunConfigurePath
            else:
                filePath = inputFilePath
            # fileName = filePath + inputFileName
            fileName = os.path.join(filePath, inputFileName)
        else:
            fileName = filename
        fp = open(fileName,'w')
        try :
            fp.write(json.dumps(result,indent=4))
            print "success"
        except IOError:
            print "Error: can\'t find file or read data"
        finally:
            fp.close()
			

if __name__ == "__main__":
    handler = EcsExportHandler()
    handler.exportInstanceToFile()




'''
 Licensed to the Apache Software Foundation (ASF) under one
 or more contributor license agreements.  See the NOTICE file
 distributed with this work for additional information
 regarding copyright ownership.  The ASF licenses this file
 to you under the Apache License, Version 2.0 (the
 "License"); you may not use this file except in compliance
 with the License.  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing,
 software distributed under the License is distributed on an
 "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
'''
__author__ = 'xixi.xxx'

import os,sys,json
parentdir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0,parentdir)
import aliyunCliParser
import response
import commandConfigure
import aliyunOpenApiData
import aliyunExtensionCliHandler
import platform

class RdsExportDBInstanceHanlder():
    def __init__(self):
        self.cmd = 'rds'
        self.operations = ['DescribeDBInstanceAttribute']
        self.parser = aliyunCliParser.aliyunCliParser()
        self.apiHandler = aliyunOpenApiData.aliyunOpenApiDataHandler()
        self.extensionHandler = aliyunExtensionCliHandler.aliyunExtensionCliHandler()

    def getFileName(self,keyValues):
        filename = None
        if keyValues.has_key('--filename') and len(keyValues['--filename']) > 0:
            filename = keyValues['--filename'][0]
        else:
            return filename, "A file name is needed! please use \'--filename\' and add the file name."
        return filename, ''

    def exportDBInstance(self, cmd, operation, version, secureRequest=False):
        rdsConfigure = commandConfigure.rds()
        if cmd.lower() == rdsConfigure.cmdName.lower() and operation.lower() ==rdsConfigure.exportDBInstance.lower():
            pass
        else :
            return None
        _keyValues = self.parser.getCliKeyValues()
        filename, errorMsg = self.getFileName(_keyValues)
        if filename is None:
            import cliError
            errorClass = cliError.error()
            errorClass.printInFormat(errorClass.CliException, errorMsg)
            return
        operation = self.operations[0]
        if self.apiHandler.isAvailableOperation(cmd, operation, version):
            cmdInstance, mclassname = self.apiHandler.getInstanceByCmdOperation(cmd, operation, version)
            if not cmdInstance is None:
                newkeyValues = self.parser.getOpenApiKeyValues(_keyValues)
                if self.apiHandler.needSetDefaultRegion(cmdInstance, newkeyValues):
                    newkeyValues["RegionId"] = [self.extensionHandler.getUserRegion()]
                self.apiHandler.setAttr(cmdInstance, newkeyValues) # set all key values in instance
                self.apiHandler.changeEndPoint(cmdInstance, newkeyValues)
                try:
                    result = self.apiHandler.getResponse(cmd,operation, mclassname, cmdInstance, newkeyValues,secureRequest)
                    self.apiHandler.responseOptimize(result,cmd,operation)
                    # result = cmdInstance.getResponse()
                    # result = self._optimizeResult(result)
                    if("Code" in result):
                        response.display_response("error", result, "json")
                    else:
                        if not filename == None:
                            self.exportInstanceToFile(result,filename)
                        else:
                            print 'Filename is needed'
                except Exception,e:
                    print(e)

    def exportInstanceToFile(self, result, filename):
        inputFilePath  = os.path.split(filename)[0]
        inputFileName = os.path.split(filename)[1]
        if not os.path.isfile(filename):
            if inputFilePath == '':
                filePath = self.extensionHandler.aliyunConfigurePath
            else:
                filePath = inputFilePath
            fileName=os.path.join(filePath,inputFileName)
            # fileName = os.path.join(filePath, inputFileName)
        else:
            fileName = filename
        fp = open(fileName,'w')
        try :
            fp.write(json.dumps(result,indent=4))
            print "success"
        except IOError:
            print "Error: can\'t find file or read data"
        finally:
            fp.close()

if __name__ == "__main__":
    handler = RdsExportDBInstanceHanlder()
    handler.exportInstanceToFile("haha", "test")




import os,sys,json
parentdir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0,parentdir)
import aliyunExtensionCliHandler
import aliyunOpenApiData
import aliyunCliParser
import response 
class EcsImportHandler:
    def __init__(self):
        self.parser = aliyunCliParser.aliyunCliParser()
        self.apiHandler = aliyunOpenApiData.aliyunOpenApiDataHandler()
        self.extensionHandler = aliyunExtensionCliHandler.aliyunExtensionCliHandler()
    
    def getFileName(self,keyValues):
        filename = None
        if keyValues.has_key('--filename') and len(keyValues['--filename']) > 0:
            filename = keyValues['--filename'][0]
        else:
            print "A profile is needed! please use \'--filename\' and add the profile name."
        return filename

    def getInstanceCount(self,keyValues):
        count = 1
        if keyValues.has_key('--instancecount') and len(keyValues['--instancecount']) > 0:
            if  keyValues['--instancecount'][0].isdigit() and int(keyValues['--instancecount'][0]) >= 0:
                count = keyValues['--instancecount'][0]
            else:
                print "InstanceCount should be a positive number! The default value(1) will be used!"
        return int(count)
    
    def getSubOperations(self,cmd,operation):
        operations = None
        if cmd.lower() == 'ecs' and operation.lower() =='importinstance':
            operations = ['CreateInstance']
        return operations 

    def _handSubOperation(self,cmd,operations,keyValues,version,_isAllocatePublicIp=False,secureRequest=False):
        if keyValues is None:
            return
        for item in operations:
            if self.apiHandler.isAvailableOperation(cmd, item, version):
                cmdInstance,mclassname = self.apiHandler.getInstanceByCmdOperation(cmd, item, version)
                if not cmdInstance is None:
                    newkeyValues = self.parser.getOpenApiKeyValues(keyValues)
                    if self.apiHandler.needSetDefaultRegion(cmdInstance, newkeyValues):
                        newkeyValues["RegionId"] = [self.extensionHandler.getUserRegion()]
                    # self._setAttr(cmdInstance, newkeyValues) # set all key values in instance
                    # self.apiHandler.changeEndPoint(cmdInstance, newkeyValues)
                    try:
                        # result = cmdInstance.getResponse()
                        result = self.apiHandler.getResponse(cmd, item, mclassname, cmdInstance, newkeyValues,secureRequest)
                        self.apiHandler.responseOptimize(result,cmd,item)
                        if("Code" in result):
                            response.display_response("error", result, "json")
                        else:
                            response.display_response(item, result, "json")
                            if _isAllocatePublicIp is True:
                                extraOperation = "AllocatePublicIpAddress"
                                instanceId = self.getInstanceIdFromJsonBody(result)
                                if instanceId is not None:
                                    key=list()
                                    _newkeyValues=dict()
                                    key.append(instanceId)
                                    _newkeyValues["InstanceId"] = key
                                    _newkeyValues["RegionId"] = newkeyValues["RegionId"]
                                    self._handExtraOperation(cmd,extraOperation,_newkeyValues,version,secureRequest)
                                else:
                                    print "InstanceId  is need!"
                    except Exception,e:
                        print(e)

    def _handExtraOperation(self,cmd,extraOperation,keyValues,version , secureRequest = False):
        if self.apiHandler.isAvailableOperation(cmd, extraOperation, version):
            cmdInstance,mclassname = self.apiHandler.getInstanceByCmdOperation(cmd, extraOperation, version)
            if  cmdInstance is not None:
                try:
                    # result = cmdInstance.getResponse()
                    result = self.apiHandler.getResponse(cmd, extraOperation, mclassname, cmdInstance, keyValues,secureRequest)
                    self.apiHandler.responseOptimize(result,cmd,extraOperation)
                    if("Code" in result):
                        response.display_response("error", result, "json")
                    else:
                        response.display_response(extraOperation, result, "json")
                except Exception,e:
                    print(e)


    def getKVFromJson(self,filename):
        if filename is None:
            return None
        inputFilePath  = os.path.split(filename)[0]
        inputFileName = os.path.split(filename)[1]
        if not os.path.isfile(filename):
            if inputFilePath == '':
                filePath = self.extensionHandler.aliyunConfigurePath
            else:
                filePath = inputFilePath
            fileName = os.path.join(filePath, filename)
        else:
            fileName = filename
        fp = open(fileName,'r')
        try:
            data=json.loads(fp.read())
            keys = data.keys()
            newMap = dict()
            for key in keys:
                value = list()
                value.append(data[key])
                newMap[key] = value
            return newMap	
        finally:
            fp.close()

    def isAllocatePublicIpAddress(self,keyValues):
        _publicIp = False
        if keyValues.has_key('--allocatepublicip') and len(keyValues['--allocatepublicip']) > 0:
            if  keyValues['--allocatepublicip'][0] == "yes":
                _publicIp = True
        return _publicIp

    def getInstanceIdFromJsonBody(self,jsonbody):
        instanceId = None
        try:
            data = jsonbody
            '''
            data = json.loads(jsonbody)
            '''
            if data.has_key('InstanceId') and len(data['InstanceId']) > 0:
                instanceId = data['InstanceId']
        except Exception,e:
            pass
        finally:
            return instanceId


    def ImportInstance(self,cmd,operation,version,secureRequest = False):
        _keyValues = self.parser.getCliKeyValues()
        _isAllocatePublicIp = self.isAllocatePublicIpAddress(_keyValues)
        operations = self.getSubOperations(cmd,operation)
        _instanceCount = self.getInstanceCount(_keyValues)
        filename = self.getFileName(_keyValues)
        keyValues = self.getKVFromJson(filename)
        for i in range(1,_instanceCount+1):
            self._handSubOperation(cmd,operations,keyValues,version,_isAllocatePublicIp,secureRequest)
# this method will set all key:value for open api class
    def _setAttr(self, classname, map):
        try:
            for key in map.keys():
                if  not key in ["AccessKeyId", "AccessKeySecret", "Endpoint"]:
                    value = map.get(key)
                    classname.__setattr__(key, value)
        except Exception as e:
            pass
 

if __name__ == "__main__":
    handler = EcsImportHandler()
    handler.getKVFromJson('ttt')
    print handler.getKVFromJson('ttt')

